#include "LinearProgramSolver.h"

int SimplexAlgorithm(const CMatrixList& mData, CMatrixList& Result)
{
	double tolerance = 0.0000001;
	int i,j,k;
	int counter,counter2;
	int mDataX, mDataY;
	int tabX,tabY;
	CMatrixList Tableau;

	mData.getDimensions(&mDataY, &mDataX);

	//Matrix not big enough
	if(mDataX<3 || mDataY<2)
		return -4;

	counter=0;
	for(i=2;i<=mDataY;i++)
	{
		double result;
		result = mData.GetValue(i,mDataX-1);

		//if equality
		if(-0.1<result && result<0.1)
			counter++;
	}

	if(!Tableau.setDimensions(mDataY+1,1+(mDataX-2)+(mDataY-1-counter)+(mDataY-1)+1))
		return -1;//ran out of memory
	
	if(!Result.setDimensions(mDataX-2,1))
		return -1;//ran out of memory

	Tableau.Fill(0);
	Result .Fill(0);

	//i=1
	if(mData.GetValue(1,mDataX-1)<-tolerance)
	{
		//Minimize
		for(j=1;j<=mDataX-2;j++)
			Tableau.SetValue(1,j+1,mData.GetValue(1,j));
	}
	else
	{
		//Maximize
		for(j=1;j<=mDataX-2;j++)
			Tableau.SetValue(1,j+1,-mData.GetValue(1,j));
	}

	counter2=1+(mDataX-2)+1;
	for(i=2;i<=mDataY;i++)
	{
		//set first/artificial columns
		Tableau.SetValue(i,1,1+(mDataX-2)+(mDataY-1-counter)+i-1-1);
		Tableau.SetValue(i,1+(mDataX-2)+(mDataY-1-counter)+i-1,1);

		//Set Normal/Slack/Last columns
		if(mData.GetValue(i,mDataX)<0)
		{
			//Normal
			for(j=1;j<=mDataX-2;j++)
				Tableau.SetValue(i,j+1,-mData.GetValue(i,j));
			
			//Slack
			double result;
			result = mData.GetValue(i,mDataX-1);
			if(result>=0.1)
			{
				Tableau.SetValue(i,counter2,-1);
				counter2++;
			}
			else
			if(result<=-0.1)
			{
				Tableau.SetValue(i,counter2,1);
				counter2++;
			}

			//Last
			Tableau.SetValue(i,1+(mDataX-2)+(mDataY-1-counter)+(mDataY-1)+1,-mData.GetValue(i,mDataX));
		}
		else
		{
			//Normal
			for(j=1;j<=mDataX-2;j++)
				Tableau.SetValue(i,j+1,mData.GetValue(i,j));
			
			//Slack
			double result;
			result = mData.GetValue(i,mDataX-1);
			if(result>=0.1)
			{
				Tableau.SetValue(i,counter2,1);
				counter2++;
			}
			else
			if(result<=-0.1)
			{
				Tableau.SetValue(i,counter2,-1);
				counter2++;
			}

			//Last
			Tableau.SetValue(i,1+(mDataX-2)+(mDataY-1-counter)+(mDataY-1)+1,mData.GetValue(i,mDataX));
		}
	}
	
	//i=mDataY+1
	for(j=1;j<=mDataY-1;j++)
		//Maximize -z1-z2-...
		Tableau.SetValue(mDataY+1,1+(mDataX-2)+(mDataY-1-counter)+j,1);

	
	Tableau.getDimensions(&tabY,&tabX);
	bool   bExistPstv;
	double result;
	double minQuotient;
	int    minRow;

	//Phase 1
	//Rewrite objective function
	for(j=2;j<=tabX;j++)
	{
		result = Tableau.GetValue(tabY,j);
		for(i=2;i<=tabY-1;i++)
			result-=Tableau.GetValue(i,j);
		Tableau.SetValue(tabY,j,result);
	}


	while(true)
	{
		bExistPstv=false;
		for(j=2;j<=tabX-1;j++)
		{
			result = Tableau.GetValue(tabY,j);
			if(result<=-tolerance)
			{
				bExistPstv=true;
				break;
			}
		}
		
		if(bExistPstv==false)
			break;//Success Phase 1 complete


		bExistPstv=false;
		for(i=2;i<=tabY-1;i++)
		{
			result = Tableau.GetValue(i,j);
			if(result>=tolerance)
			{
				result=Tableau.GetValue(i,tabX)/result;

				if(bExistPstv==true)
				{
					if(result+tolerance<minQuotient)
					{
						minQuotient = result;
						minRow      = i;
					}
				}
				else
				{
						minQuotient = result;
						minRow      = i;
				}

				bExistPstv=true;
			}
		}

		if(bExistPstv==false)
			return -2;//No Solution Unbounded

		//Pivot about minRow,j
		//"Normalize"
		result = Tableau.GetValue(minRow,j);
		for(k=2;k<=tabX;k++)
		{
			double* element;
			element = Tableau.GetPointer(minRow,k);
			*element/=result; 
		}

		//Subtract from other rows
		for(i=1;i<=tabY;i++)
		{
			if(i==minRow)
				continue;

			result = Tableau.GetValue(i,j);
			for(k=2;k<=tabX;k++)
			{
				double* element;
				element = Tableau.GetPointer(i,k);
				*element-=result*Tableau.GetValue(minRow,k); 
			}
		}

		//Set first column value
		Tableau.SetValue(minRow,1,j-1);
	}//loop Phase 1

	result = 0;
	for(i=2;i<=tabY-1;i++)
		if(Tableau.GetValue(i,1)>1+(mDataX-2)+(mDataY-1-counter)+0.3)
			result+=Tableau.GetValue(i,tabX);

	if(result>tolerance)
		return -2;//No feasible solution. Min of artificial of variables is > 0
	
	//Check if we only want to find a feasible solution
	if(mData.GetValue(1,mDataX-1)<0.5 && mData.GetValue(1,mDataX-1)>-0.5)
	{
		for(i=2;i<=tabY-1;i++)
		{
			result=Tableau.GetValue(i,1);

			if(result<=mDataX-2+0.1)
				Result.SetValue((int)(result+0.001),1,Tableau.GetValue(i,tabX));
		}

		return 1;
	}


	//Phase 2
	while(true)
	{
		bExistPstv=false;
		for(j=2;j<=tabX-1;j++)
		{
			result = Tableau.GetValue(1,j);
			if(result<=-tolerance)
			{
				bExistPstv=true;
				break;
			}
		}
		
		if(bExistPstv==false)
			break;//Success Found Optimal. Phase 2 Complete


		bExistPstv=false;
		for(i=2;i<=tabY-1;i++)
		{
			result = Tableau.GetValue(i,j);
			if(result>=tolerance)
			{
				result=Tableau.GetValue(i,tabX)/result;

				if(bExistPstv==true)
				{
					if(result+tolerance<minQuotient)
					{
						minQuotient = result;
						minRow      = i;
					}
				}
				else
				{
						minQuotient = result;
						minRow      = i;
				}

				bExistPstv=true;
			}
		}

		if(bExistPstv==false)
			return -3;//No Solution [Unbounded]

		//Pivot about minRow,j
		//"Normalize"
		result = Tableau.GetValue(minRow,j);
		for(k=2;k<=tabX;k++)
		{
			double* element;
			element = Tableau.GetPointer(minRow,k);
			*element/=result; 
		}

		//Subtract from other rows
		for(i=1;i<=tabY;i++)
		{
			if(i==minRow)
				continue;

			result = Tableau.GetValue(i,j);
			for(k=2;k<=tabX;k++)
			{
				double* element;
				element = Tableau.GetPointer(i,k);
				*element-=result*Tableau.GetValue(minRow,k); 
			}
		}

		//Set first column value
		Tableau.SetValue(minRow,1,j-1);

	}//loop Phase 2

	for(i=2;i<=tabY-1;i++)
	{
		result=Tableau.GetValue(i,1);

		if(result<=mDataX-2+0.1)
			Result.SetValue((int)(result+0.001),1,Tableau.GetValue(i,tabX));
	}

	return 1;
}

