#include "MatrixList.h"


CMatrixList::CMatrixList()
{
	dimensionX  = 0;
	dimensionY  = 0;
	Data.setDimension(0);
}


CMatrixList::~CMatrixList()
{
	dimensionX  = 0;
	dimensionY  = 0;
	Data.setDimension(0);
}


CMatrixList::CMatrixList(const CMatrixList& B)
{
	if(Data.dimension!=B.Data.dimension)
		Data.setDimension(B.Data.dimension);

	if(Data.dimension<=0)
	{
		Data.setDimension(0);
		dimensionX = 0;
		dimensionY = 0;
		return;
	}

	Data       = B.Data;
	dimensionX = B.dimensionX;
	dimensionY = B.dimensionY;
	return;
}


bool CMatrixList::setDimensions(int  y, int  x)
{
	if(x<=0 || y<=0)
	{
		Data.setDimension(0);
		dimensionX = 0;
		dimensionY = 0;
		return false;
	}

	if(Data.dimension!=x*y)
		Data.setDimension(x*y);

	if(Data.dimension<=0)
	{
		Data.setDimension(0);
		dimensionX = 0;
		dimensionY = 0;
		return false;
	}

	Data.Fill(0);
	dimensionX = x;
	dimensionY = y;

	return true;
}


void CMatrixList::getDimensions(int* y, int* x) const
{
	if(x!=NULL)
		*x = dimensionX;
	if(y!=NULL)
		*y = dimensionY;

	return;
}


//VectorList = {(1,1),(2,1),...,(dimensionX,1),(1,2),...,(dimensionX,dimensionY)}
double* CMatrixList::GetPointer(int y, int x) const
{
	if(y<=0 || x<=0 || y>dimensionY || x>dimensionX)
		return NULL;
	
	return &(Data.array[(y-1)*dimensionX+x-1]);
}


double CMatrixList::GetValue(int y, int x) const
{
	double* pElement;

	pElement=GetPointer(y,x);
	if(pElement==NULL)
		return 0;

	return (*pElement);
}


void CMatrixList::SetValue(int y, int x, double value)
{
	double* pElement;

	pElement=GetPointer(y,x);
	if(pElement==NULL)
		return;

	*pElement = value;

	return;
}


void CMatrixList::operator =(const CMatrixList& B)
{
	if(B.dimensionX<=0 || B.dimensionY<=0)
	{
		setDimensions(0,0);
		return;
	}

	if(!setDimensions(B.dimensionY,B.dimensionX))
	{
		setDimensions(0,0);
		return;
	}

	Data=B.Data;
	
	return;
}


CMatrixList CMatrixList::operator -(const CMatrixList& B) const
{	
	CMatrixList M;

	if(dimensionX!=B.dimensionX || dimensionY!=B.dimensionY || dimensionX<=0 || dimensionY<=0)
		return M;

	M.setDimensions(dimensionY,dimensionX);
	if(M.dimensionX<=0)
		return M;

	M.Data=Data;
	M.Data-=B.Data;

	return M;
}


CMatrixList CMatrixList::operator +(const CMatrixList& B) const
{	
	CMatrixList M;

	if(dimensionX!=B.dimensionX || dimensionY!=B.dimensionY || dimensionX<=0 || dimensionY<=0)
		return M;

	M.setDimensions(dimensionY,dimensionX);
	if(M.dimensionX<=0)
		return M;

	M.Data=Data;
	M.Data+=B.Data;

	return M;
}


CMatrixList CMatrixList::operator *(double lambda) const
{	
	CMatrixList M;

	if(dimensionX<=0 || dimensionY<=0)
		return M;

	M.setDimensions(dimensionY,dimensionX);
	if(M.dimensionX<=0)
		return M;

	M.Data=Data;
	M.Data*=lambda;

	return M;
}


CMatrixList CMatrixList::operator /(double lambda) const
{	
	CMatrixList M;

	if(dimensionX<=0 || dimensionY<=0)
		return M;

	M.setDimensions(dimensionY,dimensionX);
	if(M.dimensionX<=0)
		return M;

	M.Data=Data;
	M.Data/=lambda;

	return M;
}


void CMatrixList::operator +=(const CMatrixList& B)
{	
	CMatrixList M;

	if(dimensionX!=B.dimensionX || dimensionY!=B.dimensionY || dimensionX<=0 || dimensionY<=0)
		return;

	Data+=B.Data;

	return;
}


void CMatrixList::operator -=(const CMatrixList& B)
{	
	CMatrixList M;

	if(dimensionX!=B.dimensionX || dimensionY!=B.dimensionY || dimensionX<=0 || dimensionY<=0)
		return;

	Data-=B.Data;

	return;
}


void CMatrixList::operator *=(double lambda)
{	
	CMatrixList M;

	if(dimensionX<=0 || dimensionY<=0)
		return;

	Data*=lambda;

	return;
}


void CMatrixList::operator /=(double lambda)
{	
	CMatrixList M;

	if(dimensionX<=0 || dimensionY<=0)
		return;

	Data/=lambda;

	return;
}


CMatrixList CMatrixList::operator *(const CMatrixList& B) const
{
	int i,j,k;
	double result;
	CMatrixList M;

	if(dimensionX!=B.dimensionY
	||   dimensionX<=0 ||   dimensionY<=0
	|| B.dimensionX<=0 || B.dimensionY<=0)
		return M;

	M.setDimensions(dimensionY,B.dimensionX);
	if(M.dimensionX<=0)
		return M;

	for(i=1;i<=dimensionY;i++)
		for(j=1;j<=B.dimensionX;j++)
		{
			result=0;
			for(k=1;k<=dimensionX;k++)
				result+=GetValue(i,k)*B.GetValue(k,j);

			M.SetValue(i,j,result);
		}

	return M;
}


void CMatrixList::operator *=(const CMatrixList& B)
{
	int i,j,k;
	double result;
	CMatrixList M;

	if(dimensionX!=B.dimensionY
	||   dimensionX<=0 ||   dimensionY<=0
	|| B.dimensionX<=0 || B.dimensionY<=0)
		return;

	M.setDimensions(dimensionY,B.dimensionX);
	if(M.dimensionX<=0)
	{
		setDimensions(0,0);
		return;
	}

	for(i=1;i<=dimensionY;i++)
		for(j=1;j<=B.dimensionX;j++)
		{
			result=0;
			for(k=1;k<=dimensionX;k++)
				result+=GetValue(i,k)*B.GetValue(k,j);

			M.SetValue(i,j,result);
		}

	*this=M;

	return;
}


//B * v
CVectorList CMatrixList::operator *(const CVectorList& v) const
{
	int i,j;
	double result;
	CVectorList R;

	if(dimensionX!=v.dimension
	||   dimensionX<=0 ||   dimensionY<=0
	|| v.dimension<=0)
		return R;

	R.setDimension(dimensionY);
	if(R.dimension<=0)
		return R;

	for(i=1;i<=dimensionY;i++)
	{
		result=0;
		for(j=1;j<=dimensionX;j++)
			result+=GetValue(i,j)*v.array[j-1];

		R.array[i-1]=result;
	}

	return R;
}


void CMatrixList::Identity()
{
	int i;
	int min;

	if(dimensionX<=0 || dimensionY<=0)
		return;

	min=(dimensionX<dimensionY)?dimensionX:dimensionY;
	
	Fill(0);
	for(i=1;i<=min;i++)
		SetValue(i,i,1);

	return;
}


void CMatrixList::Fill(double lambda)
{
	Data.Fill(lambda);

	return;
}


void CMatrixList::Transpose()
{
	CMatrixList Transposed;

	Transposed=ReturnTranspose();

	return;
}


CMatrixList CMatrixList::ReturnTranspose() const
{
	int i,j;
	CMatrixList Transposed;

	if(dimensionX<=0 || dimensionY<=0)
		return Transposed;

	Transposed.setDimensions(dimensionX,dimensionY);
	if(Transposed.dimensionX<=0)
		return Transposed;

	for(i=1;i<=dimensionX;i++)
		for(j=0;j<=dimensionY;j++)
			Transposed.SetValue(i,j,GetValue(j,i));

	return Transposed;
}




CMatrixList operator *(double lambda, const CMatrixList& B)
{
	return CMatrixList(B*lambda);
}


//[B^T * v]^T
CVectorList operator *(const CVectorList& v, const CMatrixList& B)
{
	return CVectorList(B.ReturnTranspose()*v);
}


//[B^T * v]^T
void operator *=(CVectorList& v, const CMatrixList& B)
{
	CVectorList result(B.ReturnTranspose()*v);

	v.setDimension(result.dimension);
	v = result;

	return;
}

