#ifndef __VECTOR_LIST__
#define __VECTOR_LIST__

#include <stdlib.h>
#include <math.h>

class CVectorList
{
public:
	
	CVectorList();
	~CVectorList();
	CVectorList(const CVectorList& v);
	
	bool        setDimension(int dim);
	void        Fill(double lambda);

	//if (*this).dimension and v.dimension are not equal the functions ignore
	//values above the min of the dimensions and leaves their values intact and
	//or return a truncated CVectorList object.
	void        operator =( const CVectorList&      v);
	CVectorList operator -( const CVectorList&      v) const;
	CVectorList operator +( const CVectorList&      v) const;
	CVectorList operator *(       double       lambda) const;
	CVectorList operator /(       double       lambda) const;
	void        operator +=(const CVectorList&      v);
	void        operator -=(const CVectorList&      v);
	void        operator *=(      double       lambda);
	void        operator /=(      double       lambda);

	double      Dot(const CVectorList& v) const;
	double      Length()                  const;
	void        Normalize();
	//Computes (*this)=L[0]*V[0]+L[1]*V[1]+...
	void        LinearAddition(CVectorList** V, double* L, int dimensionOfArrays);

	double*     array;
	int         dimension;
};

CVectorList operator *(double lambda, const CVectorList& v);

#endif

/*
 /                  \
|      array[0]      |
|                    |
|      array[1]      |
|         |          |
|         V          |
| array[dimension-1] |
 \                  /

*/

