#ifndef _VIDEO_CAPTURE_H_
#define _VIDEO_CAPTURE_H_

typedef unsigned __int64 VC_DWORDLONG;
typedef unsigned short   VC_WCHAR;
typedef long             VC_HRESULT;
typedef unsigned char    VC_BYTE;

class ResourceValues
{
public:
	int m_IDD_COMPRESS_DIALOG;
	int m_IDC_COMBO;
	int m_IDC_CONFIGURE;
	int m_IDC_ABOUT;
	int m_IDOK;
	int m_IDC_SCROLLBAR;
	int m_IDC_QUALITY_TEXT;
	int m_IDC_VALUE;

	void Set(
		int s_IDD_COMPRESS_DIALOG,
		int s_IDC_COMBO,
		int s_IDC_CONFIGURE,
		int s_IDC_ABOUT,
		int s_IDOK,
		int s_IDC_SCROLLBAR,
		int s_IDC_QUALITY_TEXT,
		int s_IDC_VALUE
		);
};

class CompressorPropertiesToSet
{
public:
	CompressorPropertiesToSet();
	~CompressorPropertiesToSet();

	//Friendly Name to search for
	char*        Name;

	//What the user wants to set
	bool         bSet_KeyFrameRate;
	bool         bSet_PFramesPerKey;
	bool         bSet_Quality;
	bool         bSet_WindowSize;

	//Values to set
	long         KeyFrameRate;
	long         PFramesPerKey;
	double       Quality;
	VC_DWORDLONG WindowSize;
};

enum ClrType { RGB_1=0, RGB_4, RGB_8, RGB_444, RGB_555, RGB_565, RGB_24, RGB_32};

class VideoProperties
{
public:
	char* name;
	bool  bWriteOverIfExists;

	int   width;
	int   height;
	int   format;
	bool  bCreateBuffer;

	bool  bUseTheseSettings;
	bool  bUseCompressor;
	CompressorPropertiesToSet compressProp;
};

//Filled by IAMVideoCompression::GetInfo() method
class VideoCompressionProperties
{
public:
	VideoCompressionProperties();
	~VideoCompressionProperties();
	void Cleanup();

	VC_WCHAR*    pszVersion;
	int          cbVersion;
	VC_WCHAR*    pszDescription;
	int          cbDescription;

	//Default Values
	long         DefaultKeyFrameRate;
	long         DefaultPFramesPerKey;
	double       DefaultQuality;

	//Current Values
	long         KeyFrameRate;
	long         PFramesPerKey;
	double       Quality;
	VC_DWORDLONG WindowSize;

	//Capabilities;
	bool         bCAPCanQuality;
    bool         bCAPCanCrunch;
	bool         bCAPCanKeyFrame;
    bool         bCAPCanBFrame;
    bool         bCAPCanWindow;
};

//Return values from IAMVfwCompressDialogs::ShowDialog() method
class CompressDialogsProperties
{
public:
	bool bConfigExists;
	bool bAboutExists;
};

class CompressorProperties
{
public:
	CompressorProperties();
	~CompressorProperties();
	bool bNameFilledIn;
	char FriendlyName[256];
	int  id;//For use by GetAllCompressorInfo

	bool bInfoFilledIn;
	bool bHas_IAMVideoCompression_Interface;
	VideoCompressionProperties vcp;
	bool bHas_IAMVfwCompressDialogs_Interface;
	CompressDialogsProperties cdp;
};

//Wrapper class for CFilterGraphManager
class CVideoCapture
{
public:
	CVideoCapture();
	~CVideoCapture();

	bool       Create();
	void       Destroy();

	VC_HRESULT Init();
	void       Cleanup();

	VC_HRESULT CreateFilterGraph(VideoProperties* prop);
	void       End();
	int        GetBufferSize();

	//If value is NULL returns whether it is ready for more data
	//pData should be DWORD aligned
	bool       AddImage (VC_BYTE* pData, int repeatTime);

	//If value is NULL returns whether it is ready for more data
	//pData should be DWORD aligned
	//pData should include the header info
	bool       AddBitmap(VC_BYTE* pData, int repeatTime);

	void       DisplayCompressorDialog(void* phWndParent, VideoProperties* prop);
	void       SetResourceValues(ResourceValues* rv);
	
	bool	   InitArray();
	void       CleanupArray();
	bool       ReleaseArrayGraph();

	void       DisplayCompressorProperties(CompressorProperties* cp);
	VC_HRESULT GetAllCompressorInfo(CompressorProperties** pArray);

	bool       GetError(char** pstr);

private:
	void*      cFGM;
};

#endif