#include "3d_object.h"
#include <stdio.h>


C3dObject::C3dObject()
{
	iNumOfVerts = 0;
	iNumOfFaces = 0;
	vertList    = NULL;
	faceList    = NULL;
	name[0]     = '\0';
	name[20]    = '\0';
}


C3dObject::~C3dObject()
{
	iNumOfVerts = 0;
	iNumOfFaces = 0;

	if(vertList!=NULL)
	{
		delete [] vertList;
		vertList=NULL;
	}

	if(faceList!=NULL)
	{
		delete [] faceList;
		faceList=NULL;
	}

	name[0]     = '\0';
	name[20]    = '\0';
}


CVertex C3dObject::Transform(int i)
{
	CVertex vert;
	CVector r;

	vert.r = vertList[i].r;
	vert.g = vertList[i].g;
	vert.b = vertList[i].b;

	r.e[0] = vertList[i].x;
	r.e[1] = vertList[i].y;
	r.e[2] = vertList[i].z;

	r = data.Orientation*r + data.Position;
	
	vert.x = r.e[0];
	vert.y = r.e[1];
	vert.z = r.e[2];

	vert.Normal = data.Orientation*vertList[i].Normal;

	return vert;
}

CVector C3dObject::TransformToWorld(int i)
{
	CVector r;

	r.e[0] = vertList[i].x;
	r.e[1] = vertList[i].y;
	r.e[2] = vertList[i].z;

	r = data.Orientation*r + data.Position;

	return r;
}


//Calculates the inertia tensor and centre of mass (moves it to the origin)
//Does not assume the object is convex
//Does assume object is solid,
//            has Mass = 1,
//            the surface has no 'holes' (ie. completely encloses a region of space),
//            the surface triangle's vertices are all defined in a clockwise orientation
void CalculateSolidInertiaAndCOM(C3dObject* pMesh)
{
	int i,j;
	int p,q;
	double Volume;
	CMatrix I;
	CVector Average, COM;
	
	Average.e[0] = 0;
	Average.e[1] = 0;
	Average.e[2] = 0;

	Volume = 0;
	COM    = CVector(0,0,0);
	for(i=0;i<pMesh->iNumOfFaces;i++)
	{
		double  v;
		CVector a,b,c,d;
		CVector COMofTetrahedron;

		a = Average;

		b.e[0] = pMesh->vertList[pMesh->faceList[i].index[0]].x;
		b.e[1] = pMesh->vertList[pMesh->faceList[i].index[0]].y;
		b.e[2] = pMesh->vertList[pMesh->faceList[i].index[0]].z;

		c.e[0] = pMesh->vertList[pMesh->faceList[i].index[1]].x;
		c.e[1] = pMesh->vertList[pMesh->faceList[i].index[1]].y;
		c.e[2] = pMesh->vertList[pMesh->faceList[i].index[1]].z;

		d.e[0] = pMesh->vertList[pMesh->faceList[i].index[2]].x;
		d.e[1] = pMesh->vertList[pMesh->faceList[i].index[2]].y;
		d.e[2] = pMesh->vertList[pMesh->faceList[i].index[2]].z;

		v = ((b-a).Dot((c-a).Cross(d-a)))/6;
		COMofTetrahedron=(a+b+c+d)/4;
		Volume+=v;
		COM+=v*COMofTetrahedron;
	}
	COM/=Volume;
	printf("\nVolume:\n%10.3lf\n", Volume);
	printf("\nCentre Of Mass:\n%10.3lf %10.3lf %10.3lf\n",COM.e[0],COM.e[1],COM.e[2]);

	for(i=0;i<pMesh->iNumOfVerts;i++)
	{
		pMesh->vertList[i].x-=COM.e[0];
		pMesh->vertList[i].y-=COM.e[1];
		pMesh->vertList[i].z-=COM.e[2];
	}

	I.Zero();

	for(i=0;i<pMesh->iNumOfFaces;i++)
	{
		CVector a,b,c,d;
		CMatrix A;
		
		a.e[0] = 0;
		a.e[1] = 0;
		a.e[2] = 0;

		b.e[0] = pMesh->vertList[pMesh->faceList[i].index[0]].x;
		b.e[1] = pMesh->vertList[pMesh->faceList[i].index[0]].y;
		b.e[2] = pMesh->vertList[pMesh->faceList[i].index[0]].z;

		c.e[0] = pMesh->vertList[pMesh->faceList[i].index[1]].x;
		c.e[1] = pMesh->vertList[pMesh->faceList[i].index[1]].y;
		c.e[2] = pMesh->vertList[pMesh->faceList[i].index[1]].z;

		d.e[0] = pMesh->vertList[pMesh->faceList[i].index[2]].x;
		d.e[1] = pMesh->vertList[pMesh->faceList[i].index[2]].y;
		d.e[2] = pMesh->vertList[pMesh->faceList[i].index[2]].z;

		A.Zero();

		for(p=0;p<3;p++)
		for(q=0;q<3;q++)
		{
			A.M[p][q] = (a.e[p]+b.e[p]+c.e[p]+d.e[p])*(a.e[q]+b.e[q]+c.e[q]+d.e[q]);
			A.M[p][q] += (a.e[p]*a.e[q] + b.e[p]*b.e[q] + c.e[p]*c.e[q] + d.e[p]*d.e[q]);
		}

		A/=120;
		A*=((b-a).Dot((c-a).Cross(d-a)));

		I.M[0][0] += A.M[0][0] + A.M[1][1] + A.M[2][2];
		I.M[1][1] += A.M[0][0] + A.M[1][1] + A.M[2][2];
		I.M[2][2] += A.M[0][0] + A.M[1][1] + A.M[2][2];
		I -= A; 
	}

	I/=Volume;

	printf("\nInertia Tensor:\n");
	for(i=0;i<3;i++)
	{
		for(j=0;j<3;j++)
			printf("%10.3lf ",I.M[i][j]);
		printf("\n");
	}

	pMesh->data.InertiaTensor=I;
	pMesh->data.InverseOfTheInertiaTensor=I.ReturnInverse();
}


//Calculates the inertia tensor and centre of mass (moves it to the origin)
//Does not assume object is convex
//Does assume object is hollow and has Mass = 1
void CalculateHollowInertiaAndCOM(C3dObject* pMesh)
{
	int i,j;
	int p,q;
	double Volume;
	CMatrix I;
	CVector COM;

	Volume = 0;
	COM    = CVector(0,0,0);
	for(i=0;i<pMesh->iNumOfFaces;i++)
	{
		double  v;
		CVector b,c,d;
		CVector COMofTriangle;

		b.e[0] = pMesh->vertList[pMesh->faceList[i].index[0]].x;
		b.e[1] = pMesh->vertList[pMesh->faceList[i].index[0]].y;
		b.e[2] = pMesh->vertList[pMesh->faceList[i].index[0]].z;

		c.e[0] = pMesh->vertList[pMesh->faceList[i].index[1]].x;
		c.e[1] = pMesh->vertList[pMesh->faceList[i].index[1]].y;
		c.e[2] = pMesh->vertList[pMesh->faceList[i].index[1]].z;

		d.e[0] = pMesh->vertList[pMesh->faceList[i].index[2]].x;
		d.e[1] = pMesh->vertList[pMesh->faceList[i].index[2]].y;
		d.e[2] = pMesh->vertList[pMesh->faceList[i].index[2]].z;

		v = ((c-b).Cross(d-b)).Length()/2;
		COMofTriangle=(b+c+d)/3;
		Volume+=v;
		COM+=v*COMofTriangle;
	}
	COM/=Volume;
	printf("\nArea:\n%10.3lf\n", Volume);
	printf("\nCentre Of Mass:\n%10.3lf %10.3lf %10.3lf\n",COM.e[0],COM.e[1],COM.e[2]);

	for(i=0;i<pMesh->iNumOfVerts;i++)
	{
		pMesh->vertList[i].x-=COM.e[0];
		pMesh->vertList[i].y-=COM.e[1];
		pMesh->vertList[i].z-=COM.e[2];
	}

	I.Zero();

	for(i=0;i<pMesh->iNumOfFaces;i++)
	{
		CVector b,c,d;
		CMatrix A;

		b.e[0] = pMesh->vertList[pMesh->faceList[i].index[0]].x;
		b.e[1] = pMesh->vertList[pMesh->faceList[i].index[0]].y;
		b.e[2] = pMesh->vertList[pMesh->faceList[i].index[0]].z;

		c.e[0] = pMesh->vertList[pMesh->faceList[i].index[1]].x;
		c.e[1] = pMesh->vertList[pMesh->faceList[i].index[1]].y;
		c.e[2] = pMesh->vertList[pMesh->faceList[i].index[1]].z;

		d.e[0] = pMesh->vertList[pMesh->faceList[i].index[2]].x;
		d.e[1] = pMesh->vertList[pMesh->faceList[i].index[2]].y;
		d.e[2] = pMesh->vertList[pMesh->faceList[i].index[2]].z;

		A.Zero();

		for(p=0;p<3;p++)
		for(q=0;q<3;q++)
		{
			A.M[p][q] = (b.e[p]+c.e[p]+d.e[p])*(b.e[q]+c.e[q]+d.e[q]);
			A.M[p][q] += (b.e[p]*b.e[q] + c.e[p]*c.e[q] + d.e[p]*d.e[q]);
		}

		A/=24;
		A*=((c-b).Cross(d-b)).Length();

		I.M[0][0] += A.M[0][0] + A.M[1][1] + A.M[2][2];
		I.M[1][1] += A.M[0][0] + A.M[1][1] + A.M[2][2];
		I.M[2][2] += A.M[0][0] + A.M[1][1] + A.M[2][2];
		I -= A; 
	}

	I/=Volume;

	printf("\nInertia Tensor:\n");
	for(i=0;i<3;i++)
	{
		for(j=0;j<3;j++)
			printf("%10.3lf ",I.M[i][j]);
		printf("\n");
	}

	pMesh->data.InertiaTensor=I;
	pMesh->data.InverseOfTheInertiaTensor=I.ReturnInverse();
}

