#include "Matrix.h"


CMatrix::CMatrix()
{
	Identity();
}


CMatrix::CMatrix(const CMatrix& B)
{
	int i,j;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			M[i][j]=B.M[i][j];
}
	
	
void CMatrix::operator =(const CMatrix& B)
{
	int i,j;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			M[i][j]=B.M[i][j];
}


CMatrix CMatrix::operator -(const CMatrix& B) const
{
	int i,j;
	CMatrix R;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			R.M[i][j]=M[i][j]-B.M[i][j];
	
	return R;
}


CMatrix CMatrix::operator +(const CMatrix& B) const
{
	int i,j;
	CMatrix R;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			R.M[i][j]=M[i][j]+B.M[i][j];
	
	return R;
}


CMatrix CMatrix::operator *(const CMatrix& B) const
{
	int i,j,k;
	CMatrix R;

	R.Zero();

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			for(k=0;k<3;k++)
				R.M[i][j]+=M[i][k]*B.M[k][j];
	
	return R;
}


CMatrix CMatrix::operator *(double lambda) const
{
	int i,j;
	CMatrix R;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			R.M[i][j]=M[i][j]*lambda;
	
	return R;
}


CMatrix CMatrix::operator /(double lambda) const
{
	int i,j;
	CMatrix R;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			R.M[i][j]=M[i][j]/lambda;
	
	return R;
}


void CMatrix::operator +=(const CMatrix& B)
{
	int i,j;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			M[i][j]+=B.M[i][j];
	
	return;
}


void CMatrix::operator -=(const CMatrix& B)
{
	int i,j;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			M[i][j]-=B.M[i][j];
	
	return;
}


void CMatrix::operator *=(const CMatrix& B)
{
	CMatrix R;

	R=(*this)*B;
	*this=R;

	return;
}


void CMatrix::operator *=(double lambda)
{
	int i,j;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			M[i][j]*=lambda;
	
	return;
}


void CMatrix::operator /=(double lambda)
{
	int i,j;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			M[i][j]/=lambda;
	
	return;
}


//B * v
CVector CMatrix::operator *(const CVector& v) const
{
	CVector u;

	u.e[0] = M[0][0]*v.e[0] + M[0][1]*v.e[1] + M[0][2]*v.e[2];
	u.e[1] = M[1][0]*v.e[0] + M[1][1]*v.e[1] + M[1][2]*v.e[2];
	u.e[2] = M[2][0]*v.e[0] + M[2][1]*v.e[1] + M[2][2]*v.e[2];

	return u;
}

	
void CMatrix::Identity()
{
	Zero();
	M[0][0]=1;
	M[1][1]=1;
	M[2][2]=1;

	return;
}


void CMatrix::Zero()
{
	int i,j;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			M[i][j]=0;

	return;
}


void CMatrix::Rotation(const CVector& w)
{
	Rotation(w,w.Length());

	return;
}


void CMatrix::Rotation(CVector n, double angle)
{
	double c;
	double s;

	c=cos(angle);
	s=sin(angle);
	n.Normalize();

	M[0][0] = n.e[0]*n.e[0]*(1-c) + c;
	M[1][0] = n.e[0]*n.e[1]*(1-c) + n.e[2]*s;
	M[2][0] = n.e[0]*n.e[2]*(1-c) - n.e[1]*s;

	M[0][1] = n.e[1]*n.e[0]*(1-c) - n.e[2]*s;
	M[1][1] = n.e[1]*n.e[1]*(1-c) + c;
	M[2][1] = n.e[1]*n.e[2]*(1-c) + n.e[0]*s;

	M[0][2] = n.e[2]*n.e[0]*(1-c) + n.e[1]*s;
	M[1][2] = n.e[2]*n.e[1]*(1-c) - n.e[0]*s;
	M[2][2] = n.e[2]*n.e[2]*(1-c) + c;

	return;
}


void CMatrix::Rotation(double x, double y, double z)
{
	CVector w;

	w.e[0] = x;
	w.e[1] = y;
	w.e[2] = z;

	Rotation(w);

	return;
}


void CMatrix::Rotation(double x, double y, double z, double angle)
{
	CVector n;

	n.e[0] = x;
	n.e[1] = y;
	n.e[2] = z;

	Rotation(n,angle);

	return;
}


void CMatrix::Transpose()
{
	int i,j;
	CMatrix R;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			R.M[i][j]=M[j][i];

	(*this)=R;

	return;
}


CMatrix CMatrix::ReturnTranspose() const
{
	int i,j;
	CMatrix R;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			R.M[i][j]=M[j][i];

	return R;
}


double CMatrix::Determinant() const
{
	return 
		M[0][0]*(M[1][1]*M[2][2]-M[1][2]*M[2][1])
	   -M[0][1]*(M[1][0]*M[2][2]-M[1][2]*M[2][0])
	   +M[0][2]*(M[1][0]*M[2][1]-M[1][1]*M[2][0]);
}


void CMatrix::Inverse()
{
	CMatrix R;

	R=ReturnInverse();
	(*this)=R;
}


CMatrix CMatrix::ReturnInverse() const
{
	int i,j;
	int i1,i2,j1,j2;
	CMatrix Inverse;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
		{
			if(i==0)
				i1=1;
			else
				i1=0;

			if(i==2)
				i2=1;
			else
				i2=2;

			if(j==0)
				j1=1;
			else
				j1=0;

			if(j==2)
				j2=1;
			else
				j2=2;

			Inverse.M[j][i]=(1-2*((i+j)%2))*(M[i1][j1]*M[i2][j2]-M[i1][j2]*M[i2][j1]);
		}

	return Inverse/(*this).Determinant();
}


CMatrix operator *(double lambda, const CMatrix& B)
{
	int i,j;
	CMatrix R;

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			R.M[i][j]=B.M[i][j]*lambda;
	
	return R;
}


//[B^T * v]^T
CVector operator *(const CVector& v, const CMatrix& B)
{
	CVector u;

	u=(B.ReturnTranspose())*v;

	return u;
}


//[B^T * v]^T
void operator *=(CVector& v, const CMatrix& B)
{
	CVector u;

	u=(B.ReturnTranspose())*v;
	v=u;

	return;
}

