#ifndef __MATRIX__
#define __MATRIX__

#include "Vector.h"

class CMatrix
{
public:
	
	CMatrix();
	CMatrix(const CMatrix& B);
	

	void    operator =( const CMatrix&      B);
	CMatrix operator -( const CMatrix&      B) const;
	CMatrix operator +( const CMatrix&      B) const;
	CMatrix operator *( const CMatrix&      B) const;
	CMatrix operator *(       double   lambda) const;
	CMatrix operator /(       double   lambda) const;

	void    operator +=(const CMatrix&      B);
	void    operator -=(const CMatrix&      B);
	void    operator *=(const CMatrix&      B);
	void    operator *=(      double   lambda);
	void    operator /=(      double   lambda);

	CVector operator *(const CVector& v) const;//B * v
	

	void    Identity();
	void    Zero();

	//w     = axis of rotation and the magnitude of w tells us the angle
	//n     = represents the axis of rotation (will automatically be normalized)
	//angle = angle to rotate by (anticlockwise in radians looking down at the vector, ie eye to object = -n)
	void    Rotation(const CVector& w);
	void    Rotation(CVector n, double angle);
	void    Rotation(double x, double y, double z);
	void    Rotation(double x, double y, double z, double angle);

	void    Transpose();
	CMatrix ReturnTranspose() const;
	double  Determinant()     const;
	void    Inverse();
	CMatrix ReturnInverse()   const;
	
	double M[3][3];
};

CMatrix operator *(       double   lambda, const CMatrix& B);
CVector operator *( const CVector&      v, const CMatrix& B); //[B^T * v]^T
void    operator *=(      CVector&      v, const CMatrix& B); //[B^T * v]^T

#endif

/*

  /                           \    /   \
 |  M[0][0]  M[0][1]  M[0][2]  |  |  x  |
 |                             |  |     |
 |  M[1][0]  M[1][1]  M[1][2]  |  |  y  |
 |                             |  |     |
 |  M[2][0]  M[2][1]  M[2][2]  |  |  z  |
  \                           /    \   /

*/

