#ifndef __MATRIX_LIST__
#define __MATRIX_LIST__

#include "VectorList.h"

class CMatrixList
{
public:
	
	CMatrixList();
	~CMatrixList();
	//May fail due to a lack of memory
	//If it fails the dimensions of the matrix will be zero
	CMatrixList(const CMatrixList& B);
	
	//if (*this).dimension and v.dimension are not what is expected then the
	//functions may ignore values, leave the values intact, or fill in zeros
	//as it sees fit

	//If successful returns true and fills the matrix with zeros
	//If unsuccesful it returns false and will delete the matrix
	//   ie. the dimensions will be set to 0,0
	bool        setDimensions(int  y, int  x);
	//Either pointer may be set to NULL if that information is not required
	void        getDimensions(int* y, int* x) const;


	//Expect 1<=y<=dimensionY, 1<=x<=dimensionX see below
	//If the values do not lie in that region then it will return NULL or 0
	double*     GetPointer(int y, int x) const;
	double      GetValue  (int y, int x) const;
	void        SetValue  (int y, int x, double value);

	//Makes a (*this) a copy of B
	//If there is insufficient memory (*this) will be a 0x0 matrix
	void        operator =(const CMatrixList& B);

	//These functions will only work if the matrices are of the same dimensions
	//If they are not it will leave the matrices intact and return a 0x0
	//matrix if necessary (or if there is insufficient memory)
	CMatrixList operator -( const CMatrixList&      B) const;
	CMatrixList operator +( const CMatrixList&      B) const;
	CMatrixList operator *(       double       lambda) const;
	CMatrixList operator /(       double       lambda) const;
	void        operator +=(const CMatrixList&      B);
	void        operator -=(const CMatrixList&      B);
	void        operator *=(      double       lambda);
	void        operator /=(      double       lambda);

	//These functions will only work if the matrices/vector are of the compatible
	//dimensions
	//If there is insufficient memory the object being returned or updated will be
	//of zero dimension
	CMatrixList operator *( const  CMatrixList& B) const;
	void        operator *=(const  CMatrixList& B);
	CVectorList operator *( const  CVectorList& v) const;//B * v

	void        Identity();
	void        Fill(double lambda);

	//If there is insufficient memory the matrix will be made into a 0x0 matrix
	void        Transpose();
	//If three is insufficient memory it will return a 0x0 matrix
	CMatrixList ReturnTranspose() const;
	
	/* ---Not yet implemented--- */
	/*
	double  Determinant();
	void    Inverse();
	CMatrix ReturnInverse();
	*/

private:
	int         dimensionY;
	int         dimensionX;
	CVectorList Data;
};

//If there is insufficient memory it will return a zero dimension object
CMatrixList operator *(       double       lambda, const CMatrixList& B);
CVectorList operator *( const CVectorList&      v, const CMatrixList& B); //[B^T * v]^T
void        operator *=(      CVectorList&      v, const CMatrixList& B); //[B^T * v]^T

#endif

/*

  /                           \    /   \
 |  M[1][1]  M[1][2]--M[1][x]  |  |  x  |
 |                             |  |     |
 |  M[2][1]  M[2][2]           |  |  y  |
 |     |            \          |  |     |
 |  M[y][1]           M[y][x]  |  |  z  |
  \                           /    \   /

*/

