#include "VectorList.h"



CVectorList::CVectorList()
{
	array     = NULL;
	dimension = 0;
}


CVectorList::~CVectorList()
{
	if(array!=NULL)
		delete [] array;
	
	array     = NULL;
	dimension = 0;
}


CVectorList::CVectorList(const CVectorList& v)
{
	int i;

	array     = NULL;
	dimension = 0;

	if(dimension!=v.dimension || v.dimension<=0)
	{
		if(array!=NULL)
			delete [] array;
		array     = NULL;
		dimension = 0;
	}

	if(v.dimension<=0)
		return;

	if(dimension==0)
	{
		array     = new double[v.dimension];
		dimension = v.dimension;
		
		if(array==NULL)
		{
			//Allocation failed
			dimension = 0;
			return;
		}
	}

	for(i=0;i<dimension;i++)
		array[i]=v.array[i];

	return;
}


bool CVectorList::setDimension(int dim)
{
	if(dimension!=dim || dim<=0)
	{
		if(array!=NULL)
			delete [] array;
		array     = NULL;
		dimension = 0;
	}

	if(dim<=0)
		return true;

	if(dimension==0)
	{
		array     = new double[dim];
		dimension = dim;
		
		if(array==NULL)
		{
			//Allocation failed
			dimension = 0;
			return false;
		}
	}

	Fill(0);

	return true;
}


void CVectorList::Fill(double lambda)
{
	int i;

	if(dimension<=0)
		return;

	for(i=0;i<dimension;i++)
		array[i]=lambda;

	return;
}


void CVectorList::operator =(const CVectorList& v)
{
	int i;
	int min;

	min = (dimension<v.dimension)?dimension:v.dimension;

	if(min<=0)
		return;

	for(i=0;i<min;i++)
		array[i]=v.array[i];
}


CVectorList CVectorList::operator -(const CVectorList& v) const
{
	int i;
	int min;
	CVectorList result;

	min = (dimension<v.dimension)?dimension:v.dimension;

	if(min<=0)
	{
		result.setDimension(0);
		return result;
	}

	if(result.setDimension(min)==false)
		return result;

	for(i=0;i<min;i++)
		result.array[i]=array[i]-v.array[i];
	
	return result;
}


CVectorList CVectorList::operator +(const CVectorList& v) const
{
	int i;
	int min;
	CVectorList result;

	min = (dimension<v.dimension)?dimension:v.dimension;

	if(min<=0)
	{
		result.setDimension(0);
		return result;
	}

	if(result.setDimension(min)==false)
		return result;

	for(i=0;i<min;i++)
		result.array[i]=array[i]+v.array[i];
	
	return result;
}


CVectorList CVectorList::operator *(double lambda) const
{
	int i;
	int min;
	CVectorList result;

	min = dimension;

	if(min<=0)
	{
		result.setDimension(0);
		return result;
	}

	if(result.setDimension(min)==false)
		return result;

	for(i=0;i<min;i++)
		result.array[i]=lambda*array[i];
	
	return result;
}


CVectorList CVectorList::operator /(double lambda) const
{
	int i;
	int min;
	CVectorList result;

	min = dimension;

	if(min<=0)
	{
		result.setDimension(0);
		return result;
	}

	if(result.setDimension(min)==false)
		return result;

	for(i=0;i<min;i++)
		result.array[i]=array[i]/lambda;
	
	return result;
}


void CVectorList::operator +=(const CVectorList& v)
{
	int i;
	int min;

	min = (dimension<v.dimension)?dimension:v.dimension;

	if(min<=0)
		return;

	for(i=0;i<min;i++)
		array[i]+=v.array[i];
	
	return;
}


void CVectorList::operator -=(const CVectorList& v)
{
	int i;
	int min;

	min = (dimension<v.dimension)?dimension:v.dimension;

	if(min<=0)
		return;

	for(i=0;i<min;i++)
		array[i]-=v.array[i];
	
	return;
}


void CVectorList::operator *=(double lambda)
{
	int i;
	int min;

	min = dimension;

	if(min<=0)
		return;

	for(i=0;i<min;i++)
		array[i]*=lambda;
	
	return;
}


void CVectorList::operator /=(double lambda)
{
	int i;
	int min;

	min = dimension;

	if(min<=0)
		return;

	for(i=0;i<min;i++)
		array[i]/=lambda;
	
	return;
}


double CVectorList::Dot(const CVectorList& v) const
{
	int    i;
	int    min;
	double result;

	min = (dimension<v.dimension)?dimension:v.dimension;

	if(min<=0)
		return 0;

	result=0;
	for(i=0;i<min;i++)
		result+=array[i]*v.array[i];
	
	return result;
}


double CVectorList::Length() const
{	
	return sqrt(Dot(*this));
}


void CVectorList::Normalize()
{
	double length;

	length = Length();

	if(length == 0.0f)
		return;

	(*this)/=length;
	
	return;
}


void CVectorList::LinearAddition(CVectorList** V, double* L, int dimensionOfArrays)
{
	int i,j;
	int min;
	double temp;

	if(dimensionOfArrays<=0)
		return;

	min=dimension;
	for(i=0;i<dimensionOfArrays;i++)
		if(min>V[i]->dimension)
			min=V[i]->dimension;

	if(min<=0)
		return;

	for(i=0;i<min;i++)
	{
		temp=0;
		for(j=0;j<dimensionOfArrays;j++)
			temp+=(L[j]*(V[j]->array[i]));

		array[i]=temp;
	}
	
	return;
}


CVectorList operator *(double lambda, const CVectorList& v)
{
	return CVectorList(v*lambda);
}

