#include "FilterGraphManager.h"
#include <streams.h>
#include <olectl.h>
#include "MyCaptureSource.h"
#include <stdio.h>

class PassInfoToDialog
{
public:
	VideoProperties*     pVP;
	CFilterGraphManager* pcFGM;
};

void CentreDialog(HWND hwnd)
{
	HWND OwnerHwnd;
	RECT rcOwner,rcDlg;
	
	if(OwnerHwnd=GetParent(hwnd))
	{
		GetWindowRect(OwnerHwnd, &rcOwner); 
		GetWindowRect(hwnd,      &rcDlg);
		
		SetWindowPos(hwnd,NULL,
					 (rcOwner.right +rcOwner.left+rcDlg.left-rcDlg.right )/2,
					 (rcOwner.bottom+rcOwner.top +rcDlg.top -rcDlg.bottom)/2,
					 0,0,
					 SWP_NOSIZE|SWP_NOZORDER);

		SendMessage(hwnd,DM_REPOSITION,0,0);
	}

	return;
}
													
LRESULT CALLBACK CompressDialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int  i;
	char Buffer[260];
	bool bCanChangeQuality;
	bool bResult;
	CompressorPropertiesToSet cpts;
	CompressorProperties      cp;
	PassInfoToDialog*         pPITD;

	static int                  currentQuality;
	static SCROLLINFO           scrollInfo;
	static CFilterGraphManager* pcFGM;
	static VideoProperties*     vp;

    switch( message )										
    {
		case WM_INITDIALOG:											
			
			pPITD = (PassInfoToDialog*)lParam;
			pcFGM = pPITD->pcFGM;
			vp    = pPITD->pVP;
			
			CentreDialog(hWnd);

			SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_COMBO,CB_ADDSTRING,0,(LPARAM)"(Uncompressed)");

			if(pcFGM->noOfCompressorsInArray!=0)
				for(i=0;i<pcFGM->noOfCompressorsInArray;i++)
					SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_COMBO,CB_ADDSTRING,0,(LPARAM)(pcFGM->Name[i]));
			
			if(pcFGM->indexOfChosenCompressor==-1)
				SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_COMBO,CB_SELECTSTRING,-1,(LPARAM)"(Uncompressed)");
			else
				SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_COMBO,CB_SELECTSTRING,-1,(LPARAM)(pcFGM->Name[pcFGM->indexOfChosenCompressor]));
				
		
			SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETRANGE,0,109);

			scrollInfo.cbSize=sizeof(SCROLLINFO);
			scrollInfo.fMask=SIF_PAGE;
			scrollInfo.nPage=10;
			SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETSCROLLINFO,TRUE,(LPARAM)&scrollInfo);

			if(vp==NULL)
				EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), true);
			else
			{
				if(pcFGM->WillConnect(vp,pcFGM->indexOfChosenCompressor)==true)
				{
					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), true);
					if(pcFGM->indexOfChosenCompressor!=-1)
						pcFGM->ReleaseArrayGraph();
				}
				else
				{
					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), false);
				}
			}
			
			if(pcFGM->indexOfChosenCompressor==-1)
			{
				currentQuality=100;
				EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_CONFIGURE)   , false);
				EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_ABOUT)       , false);
				bCanChangeQuality=false;
			}
			else
			{
				currentQuality=pcFGM->QualityArray[pcFGM->indexOfChosenCompressor];
				if(pcFGM->FillCompressorProperties(
					&(pcFGM->CompressorArray[pcFGM->indexOfChosenCompressor]),
					&cp)==false)
				{
					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_CONFIGURE)   , false);
					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_ABOUT)       , false);
					bCanChangeQuality=false;
				}
				else
				{
					bCanChangeQuality=false;
					if(cp.bHas_IAMVideoCompression_Interface==true)
						if(cp.vcp.bCAPCanQuality==true)
							bCanChangeQuality=true;
						
						if(cp.bHas_IAMVfwCompressDialogs_Interface==true)
						{
							EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_CONFIGURE), cp.cdp.bConfigExists);
							EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_ABOUT)    , cp.cdp.bAboutExists );
						}
						else
						{
							EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_CONFIGURE), false);
							EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_ABOUT)    , false);
						}
						
				}
			}
			
			EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_QUALITY_TEXT), bCanChangeQuality);
			EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_VALUE)       , bCanChangeQuality);
			EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_SCROLLBAR)   , bCanChangeQuality);
			
			sprintf(Buffer,"%i %c",currentQuality,'%');
			SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_VALUE,WM_SETTEXT,0,(LPARAM)Buffer);
			SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETPOS,(WPARAM)currentQuality,bCanChangeQuality);
			
	
            return TRUE;

		case WM_HSCROLL:
			switch(LOWORD(wParam))
			{
			case SB_TOP:       //Scrolls to the upper left.
			case SB_BOTTOM:    //Scrolls to the lower right. 
 			case SB_ENDSCROLL: //Ends scroll.
				//Not sure when these are sent
				break;
 
			case SB_LINELEFT:  //Scrolls left by one unit.
				if(currentQuality>0)
					currentQuality--;
				SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETPOS,(WPARAM)currentQuality,TRUE);
				break;
 
			case SB_LINERIGHT: //Scrolls right by one unit. 
				if(currentQuality<100)
					currentQuality++;
				SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETPOS,(WPARAM)currentQuality,TRUE);
				break;

			case SB_PAGELEFT:  //Scrolls left by the width of the bar.
				currentQuality-=scrollInfo.nPage;
				if(currentQuality<0)
					currentQuality=0;
				SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETPOS,(WPARAM)currentQuality,TRUE);
				break;

			case SB_PAGERIGHT: //Scrolls right by the width of the bar. 
 				currentQuality+=scrollInfo.nPage;
				if(currentQuality>100)
					currentQuality=100;
				SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETPOS,(WPARAM)currentQuality,TRUE);
				break;

			case SB_THUMBTRACK://The user is dragging the scroll box.
			case SB_THUMBPOSITION://The user has dragged the scroll box (thumb) and released the mouse button.
				currentQuality=HIWORD(wParam);
				SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETPOS,(WPARAM)currentQuality,TRUE);
				break;

			default:
				break;
 			}
			
			sprintf(Buffer,"%i %c",currentQuality,'%');
			SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_VALUE,WM_SETTEXT,0,(LPARAM)Buffer);
			pcFGM->QualityArray[pcFGM->indexOfChosenCompressor]=currentQuality;
			
			break;

		case WM_COMMAND:									// If we clicked on anything in the dialog box
            
			if(LOWORD(wParam)==pcFGM->RV.m_IDOK)						// Check the LOWORD of the wParam (Which holds the ID of what was clicked on)
            {
				EndDialog(hWnd,TRUE);
				return TRUE;
			}

			if(LOWORD(wParam)==pcFGM->RV.m_IDC_ABOUT)
			{
				if(vp!=NULL)
					bResult=pcFGM->WillConnect(vp,pcFGM->indexOfChosenCompressor);
				else
					bResult=false;
				
				pcFGM->ShowDialog(
					&(pcFGM->CompressorArray[pcFGM->indexOfChosenCompressor]),
					hWnd,true,false);

				if(bResult==true)
					pcFGM->ReleaseArrayGraph();
				
				if(vp==NULL)
					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), true);
				else
				{
					if(pcFGM->WillConnect(vp,pcFGM->indexOfChosenCompressor)==true)
					{
						EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), true);
						if(pcFGM->indexOfChosenCompressor!=-1)
							pcFGM->ReleaseArrayGraph();
					}
					else
					{
						EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), false);
					}
				}

				return TRUE;
			}

			if(LOWORD(wParam)==pcFGM->RV.m_IDC_CONFIGURE)
			{
				if(vp!=NULL)
					bResult=pcFGM->WillConnect(vp,pcFGM->indexOfChosenCompressor);
				else
					bResult=false;
				
				pcFGM->ShowDialog(
					&(pcFGM->CompressorArray[pcFGM->indexOfChosenCompressor]),
					hWnd,false,true);

				if(bResult==true)
					pcFGM->ReleaseArrayGraph();
				
				if(vp==NULL)
					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), true);
				else
				{
					if(pcFGM->WillConnect(vp,pcFGM->indexOfChosenCompressor)==true)
					{
						EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), true);
						if(pcFGM->indexOfChosenCompressor!=-1)
							pcFGM->ReleaseArrayGraph();
					}
					else
					{
						EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), false);
					}
				}

				return TRUE;
			}

			if(LOWORD(wParam)==pcFGM->RV.m_IDC_COMBO)
			{
				switch(HIWORD(wParam))
				{
				case CBN_SELENDOK:
					i=SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_COMBO,CB_GETCURSEL,0,0);
					SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_COMBO,CB_GETLBTEXT,(WPARAM)i,(LPARAM)Buffer);
					
					if(strcmp(Buffer,"(Uncompressed)")==0)
						pcFGM->indexOfChosenCompressor=-1;
					else
						for(i=0;i<pcFGM->noOfCompressorsInArray;i++)
							if(strcmp(Buffer,pcFGM->Name[i])==0)
							{
								pcFGM->indexOfChosenCompressor=i;
								break;
							}

					if(vp==NULL)
						EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), true);
					else
					{
						if(pcFGM->WillConnect(vp,pcFGM->indexOfChosenCompressor)==true)
						{
							EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), true);
							if(pcFGM->indexOfChosenCompressor!=-1)
								pcFGM->ReleaseArrayGraph();
						}
						else
						{
							EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDOK), false);
						}
					}
					
					if(pcFGM->indexOfChosenCompressor==-1)
					{
						currentQuality=100;
						EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_CONFIGURE)   , false);
						EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_ABOUT)       , false);
						bCanChangeQuality=false;
					}
					else
					{
						currentQuality=pcFGM->QualityArray[pcFGM->indexOfChosenCompressor];
						if(pcFGM->FillCompressorProperties(
							&(pcFGM->CompressorArray[pcFGM->indexOfChosenCompressor]),
							&cp)==false)
						{
							EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_CONFIGURE)   , false);
							EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_ABOUT)       , false);
							bCanChangeQuality=false;
						}
						else
						{
							bCanChangeQuality=false;
							if(cp.bHas_IAMVideoCompression_Interface==true)
								if(cp.vcp.bCAPCanQuality==true)
									bCanChangeQuality=true;

							if(cp.bHas_IAMVfwCompressDialogs_Interface==true)
							{
								EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_CONFIGURE), cp.cdp.bConfigExists);
								EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_ABOUT)    , cp.cdp.bAboutExists );
							}
							else
							{
								EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_CONFIGURE), false);
								EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_ABOUT)    , false);
							}
							
						}
					}

					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_QUALITY_TEXT), bCanChangeQuality);
					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_VALUE)       , bCanChangeQuality);
					EnableWindow(GetDlgItem(hWnd,pcFGM->RV.m_IDC_SCROLLBAR)   , bCanChangeQuality);

					sprintf(Buffer,"%i %c",currentQuality,'%');
					SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_VALUE,WM_SETTEXT,0,(LPARAM)Buffer);
					SendDlgItemMessage(hWnd,pcFGM->RV.m_IDC_SCROLLBAR,SBM_SETPOS,(WPARAM)currentQuality,bCanChangeQuality);
				}
				return TRUE;
			}

	
            break;

        case WM_CLOSE:										// If we close the dialog box

			EndDialog( hWnd, FALSE );						// Close the dialog box
            
			break;

		case WM_DESTROY:									// This message happens when the dialog box closes          
        
			// If we need to free anything, do it here

			break;											// Break from the loop
    }

    return FALSE;											// Return a default false
}

void CFilterGraphManager::SetResourceValues(ResourceValues* pRV)
{
	RV.Set(
		pRV->m_IDD_COMPRESS_DIALOG,
		pRV->m_IDC_COMBO,
		pRV->m_IDC_CONFIGURE,
		pRV->m_IDC_ABOUT,
		pRV->m_IDOK,
		pRV->m_IDC_SCROLLBAR,
		pRV->m_IDC_QUALITY_TEXT,
		pRV->m_IDC_VALUE
		);
}

void CFilterGraphManager::DisplayCompressorDialog(HWND hWndParent, VideoProperties* prop)
{
	if(GraphRunning==1)
		return;

	if(CoInit==false)
		return;

	PassInfoToDialog pitd;

	pitd.pcFGM=this;
	pitd.pVP=NULL;

	if(CheckVideoProperties(prop)==true)
		pitd.pVP=prop;

	DialogBoxParam(GetModuleHandle(NULL),MAKEINTRESOURCE(RV.m_IDD_COMPRESS_DIALOG),hWndParent,
		(DLGPROC)CompressDialogProc,(LPARAM)&pitd);
}
///////////////////////////////////////////////////////
///////////////////////////////////////////////////////   

bool CFilterGraphManager::WillConnect(VideoProperties* prop, int id)
{
	HRESULT hr;

	if(CheckVideoProperties(prop)==false)
		return false;

	if(id==-1)
		return true;

	if(id>=noOfCompressorsInArray || id<-1)
		return false;

	if(GraphRunning==1)
		return false;

	ReleaseArrayGraph();
	GraphRunning=2;
	
	if(CoInit==false)
	{
		GraphRunning = 0;
		return false;
	}


	GraphRunning = 2;

	//Create the filter graph manager
	hr=CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, 
		IID_IGraphBuilder, (void **)&pGraph);
	if (FAILED(hr))
	{
		GraphRunning = 0;
		return false;
	}

	//Create the Source Filter
	//Old Comment -  Scope to hide pF.
	//CBouncingBall *
	CMyCaptureSource *pFTemp = new CMyCaptureSource(NULL,&hr);
	if (!pFTemp || FAILED(hr))
    {
		ReleaseArrayGraph();
        return false;
    }

	//Get the Source Filter's Interface
	hr=pFTemp->QueryInterface(IID_IBaseFilter, 
		reinterpret_cast<void**>(&pSource));
	if(FAILED(hr))
    {
		ReleaseArrayGraph();
        return false;
    }

	//Set Video Size and type
	pFTemp->SetDetails(prop->format,prop->width,prop->height,prop->bCreateBuffer);

	//Store the Filter object so we can access its methods 
	pF = (void*)pFTemp;

	//Add the Source to the Graph
	hr=pGraph->AddFilter(pSource,NULL);
    if (FAILED(hr))
    {
		ReleaseArrayGraph();
        return false;
    }

	//Add the compressor
	if(CompressorArray[id]==NULL)
	{
		ReleaseArrayGraph();
        return false;
	}

	hr=pGraph->AddFilter(CompressorArray[id],NULL);
	if(FAILED(hr))
	{
		ReleaseArrayGraph();
        return false;
	}

	hr=ConnectFilters(pGraph,pSource,CompressorArray[id]);
	if(SUCCEEDED(hr))
	{
		//ReleaseArrayGraph();
        return true;
	}

	ReleaseArrayGraph();
	return false;
}

bool CFilterGraphManager::InitArray(VideoProperties* prop, bool bHasToConnectToSource)
{
	int i,j;
	int Found;
	HRESULT hr;

	if(GraphRunning==1)
		return false;

	if(bHasToConnectToSource==true)
		if(CheckVideoProperties(prop)==false)
			return false;

	CleanupArray();

	if(CoInit==false)
		return false;

	CompressorProperties* Array=NULL;
	if(FAILED(GetAllCompressorInfo(&Array)))
		return false;


	GraphRunning=2;

	noOfCompressorsInArray = Array[0].id;

	CompressorArray = new IBaseFilter*[noOfCompressorsInArray];
	if(CompressorArray == NULL)
	{
		CleanupArray();
		delete [] Array;
		return false;
	}

	for(i=0;i<noOfCompressorsInArray;i++)
		CompressorArray[i]=NULL;

	Name = new char*[noOfCompressorsInArray];
	if(Name == NULL)
	{
		CleanupArray();
		delete [] Array;
		return false;
	}

	for(i=0;i<noOfCompressorsInArray;i++)
		Name[i]=NULL;

	QualityArray = new int[noOfCompressorsInArray];
	if(QualityArray == NULL)
	{
		CleanupArray();
		delete [] Array;
		return false;
	}

	for(i=0;i<noOfCompressorsInArray;i++)
		QualityArray[i]=100;


	Found=0;
	for(i=0;i<Array[0].id;i++)
	{
		hr=FindVideoCompressor(&(CompressorArray[Found]),Array[i].FriendlyName);
		if (SUCCEEDED(hr))
		{
			if(bHasToConnectToSource==true)
			{
				if(WillConnect(prop,Found)==true)
				{
					GraphRunning=2;
					ReleaseArrayGraph();
					GraphRunning=2;
					
					if(Array[i].FriendlyName==NULL)
					{
						CleanupArray();
						delete [] Array;
						return false;
					}
					
					for(j=0;j<256;j++)
					{
						if(Array[i].FriendlyName[j]=='\0')
							break;
					}
					
					Name[Found]=new char[j+1];
					if(Name[Found]==NULL)
					{	
						CleanupArray();
						delete [] Array;
						return false;
					}
					
					for(j=0;j<256;j++)
					{
						if(Array[i].FriendlyName[j]=='\0')
							break;
						else
							(Name[Found])[j] = Array[i].FriendlyName[j];
					}
					
					(Name[Found])[j] = '\0';
					
					QualityArray[Found] = 100;
					if(Array[i].bInfoFilledIn==true)
						if(Array[i].bHas_IAMVideoCompression_Interface==true)
							QualityArray[Found]=(int)((Array[i].vcp.DefaultQuality)*100);
						
					Found++;
				}//Connection succeeded
				else
				{
					CompressorArray[Found]->Release();
					CompressorArray[Found]=NULL;
				}

			}
			else//Don't need to check if it connects
			{
				if(Array[i].FriendlyName==NULL)
				{
					CleanupArray();
					delete [] Array;
					return false;
				}

				for(j=0;j<256;j++)
					if(Array[i].FriendlyName[j]=='\0')
						break;

				Name[Found]=new char[j+1];
				if(Name[Found]==NULL)
				{	
					CleanupArray();
					delete [] Array;
					return false;
				}

				for(j=0;j<256;j++)
					if(Array[i].FriendlyName[j]=='\0')
						break;
					else
						(Name[Found])[j] = Array[i].FriendlyName[j];

				(Name[Found])[j] = '\0';

				QualityArray[Found] = 100;
				if(Array[i].bInfoFilledIn==true)
					if(Array[i].bHas_IAMVideoCompression_Interface==true)
						QualityArray[Found]=(int)((Array[i].vcp.DefaultQuality)*100);

				Found++;
			}
		}
		//Failed to find the named compressor
		
		//Try the next one specified by Array[]
	}

	noOfCompressorsInArray=Found;

	delete [] Array;
	ReleaseArrayGraph();

	return true;
}

void CFilterGraphManager::CleanupArray()
{
	ReleaseArrayGraph();

	if(GraphRunning==1)
		return;

	if(noOfCompressorsInArray!=0)
	{
		for(int i=0;i<noOfCompressorsInArray;i++)
		{
			if(CompressorArray!=NULL)
			{
				if(CompressorArray[i]!=NULL)
					CompressorArray[i]->Release();
			}

			if(Name!=NULL)
			{
				if(Name[i]!=NULL)
					delete [] (Name[i]);
			}
		}
	}
		
	if(CompressorArray!=NULL)
		delete [] CompressorArray;
	CompressorArray=NULL;
	
	if(Name!=NULL)
		delete [] Name;
	Name=NULL;
	
	if(QualityArray!=NULL)
		delete [] QualityArray;
	QualityArray=NULL;

	noOfCompressorsInArray  =  0;
	indexOfChosenCompressor = -1;

	return;
}

bool CFilterGraphManager::ReleaseArrayGraph()
{
	if(GraphRunning==2)
	{
		if(pGraph!=NULL)
		{
			pGraph->Release();
			pGraph=NULL;
		}

		if(pSource!=NULL)
		{
			pSource->Release();
			pSource=NULL;
		}

		GraphRunning=0;
		return true;
	}
	else
		return false;
}

void CFilterGraphManager::DisplayCompressorProperties(CompressorProperties* prop)
{	
	if(prop->bNameFilledIn==false)
	{
		printf("--- NO NAME?\n");
	}
	else
	{
		printf("--- %s\n",prop->FriendlyName);
	}

	if(prop->bInfoFilledIn==false)
	{
		printf("\nNo Info\n\n\n");
		return;
	}

	if(!prop->bHas_IAMVideoCompression_Interface && !prop->bHas_IAMVfwCompressDialogs_Interface)
	{
		printf("\nNo Info\n\n\n");
		return;
	}


	if(prop->bHas_IAMVfwCompressDialogs_Interface)
	{
		printf("\n");
		printf("CompressDialogsInfo\n");
		printf("-------------------\n\n");

		printf("About  box : ");
		if(prop->cdp.bAboutExists)
			printf("*");
		printf("\n");

		printf("Config box : ");
		if(prop->cdp.bConfigExists)
			printf("*");
		printf("\n");
	}
	else
	{
		;//printf("Does not exist\n");
	}
		
	printf("\n");
	//printf("VideoCompressionInfo\n");
	//printf("--------------------\n\n");
	if(prop->bHas_IAMVideoCompression_Interface)
	{
		/*printf("Size of string : %i\n",prop->vcp.cbDescription);

		printf("Description    : ");
		if(prop->vcp.pszDescription==NULL)
			printf("NULL\n");
		else
			printf("%ws\n",prop->vcp.pszDescription);
		
		printf("Size of string : %i\n",prop->vcp.cbVersion);

		printf("Version        : ");
		if(prop->vcp.pszVersion==NULL)
			printf("NULL\n");
		else
			printf("%ws\n",prop->vcp.pszVersion);


		printf("\n");*/

		printf("Quality  : ");
		if(prop->vcp.bCAPCanQuality)
			printf("* %.2lf",prop->vcp.Quality);
		else
			printf("      ");
		printf(" %.2lf\n",prop->vcp.DefaultQuality);
		
		printf("Crunch   : ");
		if(prop->vcp.bCAPCanCrunch)
			printf("*");
		else
			printf(" ");
		printf("\n");
		
		printf("KeyFrame : ");
		if(prop->vcp.bCAPCanKeyFrame)
			printf("* %4i",prop->vcp.KeyFrameRate);
		else
			printf("      ");
		printf(" %4i\n",prop->vcp.DefaultKeyFrameRate);
		
		printf("BFrame   : ");
		if(prop->vcp.bCAPCanBFrame)
			printf("* %4i",prop->vcp.PFramesPerKey);
		else
			printf("      ");
		printf(" %4i\n",prop->vcp.DefaultPFramesPerKey);
		
		printf("Window   : ");
		if(prop->vcp.bCAPCanWindow)
			printf("* %4i",prop->vcp.WindowSize);
		else
			printf(" ");
		printf("\n");
	}
	else
	{
		printf("Does not exist\n");
	}

	printf("\n\n");

	return;
}

HRESULT CFilterGraphManager::GetAllCompressorInfo(CompressorProperties** pArray)
{
	// Create the System Device Enumerator.
	HRESULT hr;
	ICreateDevEnum *pSysDevEnum = NULL;
	
	if(CoInit==false)
		return E_FAIL;

	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER,
		IID_ICreateDevEnum, (void **)&pSysDevEnum);
	if (FAILED(hr))
	{
		return hr;
	}
	
	// Obtain a class enumerator for the video compressor category.
	IEnumMoniker *pEnumCat = NULL;
	hr = pSysDevEnum->CreateClassEnumerator(CLSID_VideoCompressorCategory, &pEnumCat, 0);
	
	if (hr == S_OK) 
	{
		// Enumerate the monikers.
		IMoniker *pMoniker = NULL;
		ULONG cFetched;

		int numberOfCompressorsFound=0;
		while(pEnumCat->Next(1, &pMoniker, &cFetched) == S_OK)
		{
			IPropertyBag *pPropBag;
			hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, 
				(void **)&pPropBag);
			if (SUCCEEDED(hr))
			{
				// To retrieve the filter's friendly name, do the following:
				VARIANT varName;
				VariantInit(&varName);
				hr = pPropBag->Read(L"FriendlyName", &varName, 0);
				if (SUCCEEDED(hr) && varName.vt==VT_BSTR)
					numberOfCompressorsFound++;

				VariantClear(&varName);
				pPropBag->Release();
			}
			pMoniker->Release();
		}

		if(pArray==NULL || (*pArray)!=NULL)
		{
			pEnumCat->Release();
			pSysDevEnum->Release();
			return E_FAIL;
		}

		if(numberOfCompressorsFound<1)
		{
			pEnumCat->Release();
			pSysDevEnum->Release();
			return E_FAIL;
		}

		*pArray = new CompressorProperties[numberOfCompressorsFound];

		if(*pArray==NULL)
		{
			pEnumCat->Release();
			pSysDevEnum->Release();
			return E_FAIL;
		}

		if(FAILED(pEnumCat->Reset()))
		{
			pEnumCat->Release();
			pSysDevEnum->Release();
			delete [] (*pArray);
			return E_FAIL;
		}

		int current=-1;
		//Fill in the array
		while(pEnumCat->Next(1, &pMoniker, &cFetched) == S_OK)
		{
			IPropertyBag *pPropBag;
			hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, 
				(void **)&pPropBag);
			if (SUCCEEDED(hr))
			{
				// To retrieve the filter's friendly name, do the following:
				VARIANT varName;
				VariantInit(&varName);
				hr = pPropBag->Read(L"FriendlyName", &varName, 0);
				if (SUCCEEDED(hr))
				{
					// Display the name in your UI somehow.
					if(varName.vt==VT_BSTR)
					{
						IBaseFilter* pFilter;

						current++;
						sprintf((*pArray)[current].FriendlyName,"%ws",varName.bstrVal);
						(*pArray)[current].bNameFilledIn=true;
						(*pArray)[current].id=numberOfCompressorsFound-current;

						hr = pMoniker->BindToObject(NULL, NULL, IID_IBaseFilter,
								(void**)&pFilter);

						if(FillCompressorProperties(&pFilter,&((*pArray)[current]))==true)
							(*pArray)[current].bInfoFilledIn=true;
						else
							(*pArray)[current].bInfoFilledIn=false;

						pFilter->Release();
					}
				}
				VariantClear(&varName);
				pPropBag->Release();
			}
			pMoniker->Release();

			if((*pArray)[current].id==1)
				break;
		}
		pEnumCat->Release();
		pSysDevEnum->Release();

		return S_OK;
	}
	pSysDevEnum->Release();

	return E_FAIL;
}




void CFilterGraphManager::End()
{	
	long StartTime;
	long evCode, param1, param2;
	bool bComplete;

	if(HaveFilterGraph==true)
	{
		if(bEndCalled==true)
			return;

		((CMyCaptureSource*)pF)->End();

		bComplete = false;
		StartTime = GetTickCount();
		
		while(bComplete==false && GetTickCount()-StartTime<100)
		{
			// Get all the events, and see we're done.
			while (SUCCEEDED(pEvent->GetEvent(&evCode, &param1, &param2, 0)))
			{
				pEvent->FreeEventParams(evCode, param1, param2);
				switch(evCode)
				{
				case EC_USERABORT:
				case EC_ERRORABORT:
				case EC_COMPLETE:
					bComplete = true;
					break;
				}
			}
		}

		pControl->Stop(); // Important! You must stop the graph!
		bEndCalled=true;

		return;
	}

	UpdateError("Could not End. The Filter Graph does not exist\n");
}

int  CFilterGraphManager::GetBufferSize()
{
	if(HaveFilterGraph==true)
		return ((CMyCaptureSource*)pF)->GetBufferSize();

	UpdateError("Could not get buffer size. The Filter Graph does not exist\n");
	return 0;
}

bool CFilterGraphManager::AddImage(BYTE* pData, int repeatTime)
{
	if(HaveFilterGraph==true)
	{
		if(bEndCalled==true)
		{
			UpdateError("Could not add image. The stream has ended\n");
			return false;
		}

		if(repeatTime<=0)
		{
			return ((CMyCaptureSource*)pF)->AddImage(NULL,0);
		}

		return ((CMyCaptureSource*)pF)->AddImage(pData,repeatTime);
	}

	UpdateError("Could not add image. The Filter Graph does not exist\n");
	return false;
}

bool CFilterGraphManager::AddBitmap(BYTE* pData, int repeatTime)
{
	BYTE* current;

	// Make sure we got a legal file name passed in
	if(pData == NULL)
		return AddImage(NULL,repeatTime);

	current=pData;

	// These structures will hold our bitmap file header (top of bitmap)
	// and our bitmap info header (directly underneath bitmap file header)
	BITMAPFILEHEADER bmp_fileheader = {0};
	BITMAPINFOHEADER bmp_infoheader = {0};

	// Read the file header -- Making sure the read is
	memcpy(&bmp_fileheader,current,sizeof(BITMAPFILEHEADER));
	current+=sizeof(BITMAPFILEHEADER);

	// Check the type field to make sure we have a .bmp file -- This will always be "BM" for .bmps
	if(memcmp(&bmp_fileheader.bfType, "BM", 2))
		return false;

	// Read the info header -- You'll notice it's positioned directly after the 
	// BITMAPFILEHEADER.  That's roughly how a bitmap is laid out:  BITMAPFILEHEADER
	//																BITMAPINFOHEADER
	//																Pixel Data
	memcpy(&bmp_infoheader,current,sizeof(BITMAPINFOHEADER));
	current+=sizeof(BITMAPINFOHEADER);

	// It's always good to double check and make sure the "read" went as planned :)
	// Obviously, these two numbers better be equal or there was a problem
	if(bmp_infoheader.biSize != sizeof(BITMAPINFOHEADER))
		return false;

	// This had better equal one (MSDN says this will always equal one)
	if(bmp_infoheader.biPlanes != 1)
		return false;

	// If the bitmap isn't a 16,24,32-bit bitmap, WE CAN'T LOAD IT
	// .biBitCount represents the number of bits per pixel our bitmap is
	if(
		   bmp_infoheader.biBitCount != 16
		&& bmp_infoheader.biBitCount != 24
		&& bmp_infoheader.biBitCount != 32
	  )
		return false;	

	// Read in the bitmap is compressed (has been saved with Run Length Encoding)
	// We can't load it
	if(
		   !(bmp_infoheader.biCompression == BI_RGB && (bmp_infoheader.biBitCount==24 || bmp_infoheader.biBitCount==32))
		&& !(bmp_infoheader.biCompression == BI_BITFIELDS && bmp_infoheader.biBitCount==16)
	  )
		return false;

	// Jump to the location where the bitmap data is stored
	// bfOffBits is the number of bytes from the beginning of the file to where the bitmaps bits reside 
	current=pData+bmp_fileheader.bfOffBits;

	return AddImage(current,repeatTime);
}



void CFilterGraphManager::UpdateError(char* str)
{
	Error=true;
    LengthOfStr+=sprintf(ErrorString+LengthOfStr,str);
		
    return;
}

CFilterGraphManager::CFilterGraphManager()
{
	CoInit          = false;
	HaveFilterGraph = false;
	bEndCalled      = false;

	pGraph          = NULL;
	
	pControl        = NULL;
	pEvent          = NULL;
	
	pSource         = NULL;
	pCompressor     = NULL;
	pAviMux         = NULL;
	pFileWriter     = NULL;
	pAudioRenderer  = NULL;


	GraphRunning = 0;//No graph

	CompressorArray = NULL;
	QualityArray    = NULL;
	Name            = NULL;
	noOfCompressorsInArray  = 0;
	indexOfChosenCompressor = -1;//Uncompressed


	Error=false;
	LengthOfStr = sprintf(ErrorString,"\n");
    if (FAILED(CoInitialize(NULL)))
    {
        UpdateError("Could not initialize COM library\n");
        return;
    }

	CoInit=true;
	return;
}

HRESULT CFilterGraphManager::Init()
{
	if(GraphRunning==0 || GraphRunning==1)
	{
		Error=false;
		LengthOfStr = sprintf(ErrorString,"\n");
		if (CoInit==false)
		{
			UpdateError("COM library not initialized\n");
			return E_FAIL;
		}
		
		Cleanup();
		GraphRunning = 1;

		HaveFilterGraph = false;
		bEndCalled      = false;
		
		pGraph          = NULL;
		
		pControl        = NULL;
		pEvent          = NULL;
		
		pSource         = NULL;
		pCompressor     = NULL;
		pAviMux         = NULL;
		pFileWriter     = NULL;
		pAudioRenderer  = NULL;
		
		return S_OK;
	}
	else
	{
		UpdateError("Compressor Properties is still using the Graph\n");
		return E_FAIL;
	}

	return E_FAIL;
}

CFilterGraphManager::~CFilterGraphManager()
{
	Cleanup();
	CleanupArray();

    if(CoInit==true)
	{
		CoUninitialize();
		CoInit=false;
	}
}

void CFilterGraphManager::Cleanup()
{
	if(GraphRunning!=1)
		return;

	if(HaveFilterGraph==true && bEndCalled==false)
		End();

	if(pSource != NULL)
	{
		pSource->Release();
		pSource =NULL;
	}

	if(pCompressor != NULL)
	{
		pCompressor->Release();
		pCompressor =NULL;
	}

	if(pAviMux != NULL)
	{
		pAviMux->Release();
		pAviMux =NULL;
	}

	if(pFileWriter != NULL)
	{
		pFileWriter->Release();
		pFileWriter =NULL;
	}

	if(pAudioRenderer != NULL)
	{
		pAudioRenderer->Release();
		pAudioRenderer =NULL;
	}

	if(pControl != NULL)
	{
		pControl->Release();
		pControl =NULL;
	}

	if(pEvent != NULL)
	{
		pEvent->Release();
		pEvent =NULL;
	}

	if(pGraph != NULL)
	{
		pGraph->Release();
		pGraph =NULL;
	}


	HaveFilterGraph=false;
	bEndCalled     =false;

	GraphRunning=0;

	return;
}

HRESULT CFilterGraphManager::CreateFilterGraph(VideoProperties* prop)
{
	HRESULT hr;
    IMediaFilter    *pMediaFilter = NULL;
	IFileSinkFilter	*pSink        = NULL;

	if(CheckVideoProperties(prop)==false)
	{
		UpdateError("Video Properties were not valid\n");
		Cleanup();
		return E_FAIL;
	}

	if(prop==NULL)
	{
		UpdateError("Video Properties were not valid\n");
		Cleanup();
		return E_FAIL;
	}

	if(CoInit==false)
	{
		UpdateError("Could not create Filter Graph as COM library is not initialized\n");
		Cleanup();
		return E_FAIL;
	}

	if(HaveFilterGraph)
		return S_OK;

	if(GraphRunning!=1 && GraphRunning!=0)
	{
		UpdateError("The Filter Graph is still being used by Compressor Properties\n");
		Cleanup();
		return E_FAIL;
	}

	GraphRunning=1;

    // Create the filter graph manager and query for interfaces.
    hr=CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, 
		IID_IGraphBuilder, (void **)&pGraph);
    if (FAILED(hr))
    {
        UpdateError("Could not create the Filter Graph Manager\n");
		Cleanup();
        return E_FAIL;
    }
	
    hr=pGraph->QueryInterface(IID_IMediaControl, (void **)&pControl);
    if (FAILED(hr))
    {
        UpdateError("Could not obtain the Control Interface\n");
		Cleanup();
        return E_FAIL;
    }

    hr=pGraph->QueryInterface(IID_IMediaEvent, (void **)&pEvent);
    if (FAILED(hr))
    {
        UpdateError("Could not obtain the Event Interface\n");
		Cleanup();
        return E_FAIL;
    }

	//Create the Source Filter
	//Old Comment -  Scope to hide pF.
	//CBouncingBall *
	CMyCaptureSource *pFTemp = new CMyCaptureSource(NULL,&hr);
	if (!pFTemp || FAILED(hr))
    {
        UpdateError("Could not create the Source Filter\n");
		Cleanup();
        return E_FAIL;
    }

	//Get the Source Filter's Interface
	hr=pFTemp->QueryInterface(IID_IBaseFilter, 
		reinterpret_cast<void**>(&pSource));
	if(FAILED(hr))
    {
        UpdateError("Could not obtain the Source Filter's Interface\n");
		Cleanup();
        return E_FAIL;
    }

	//Set Video Size and type
	pFTemp->SetDetails(prop->format,prop->width,prop->height,prop->bCreateBuffer);

	//Store the Filter object so we can access its methods 
	pF = (void*)pFTemp;


	//Get the Video Compressor
	if(prop->bUseTheseSettings==true)
	{
		if(prop->bUseCompressor==true)
		{
			if(prop->compressProp.Name==NULL)
			{
				UpdateError("The Compressor's Name was not specified\n");
				Cleanup();
				return E_FAIL;
			}
			
			hr=FindVideoCompressor(&pCompressor,prop->compressProp.Name);
			if (FAILED(hr))
			{
				UpdateError("The Specified Compressor could not be found\n");
				Cleanup();
				return E_FAIL;
			}
			
			//Set the compressors properties
			SetCompressorProperties(&pCompressor,&(prop->compressProp));
			
			//Used to check if the properties were set
			/*
			printf("*******************\n");
			CompressorProperties cp;
			if(FillCompressorProperties(&pCompressor,&cp))
			cp.bInfoFilledIn=true;
			DisplayCompressorProperties(&cp);
			printf("*******************\n");
			*/
		}
	}
	else
	{
		if(indexOfChosenCompressor!=-1)
		{
			CompressorPropertiesToSet cpts;
			cpts.bSet_Quality=true;
			cpts.Quality=(double)(QualityArray[indexOfChosenCompressor]/100.0);
			SetCompressorProperties(&(CompressorArray[indexOfChosenCompressor]),&cpts);
		}
	}

	//Add the Filters to the Graph
	hr=pGraph->AddFilter(pSource,NULL);
    if (FAILED(hr))
    {
        UpdateError("The Source Filter could not be added to the Filter Graph\n");
		Cleanup();
        return E_FAIL;
    }
	
	if(prop->bUseTheseSettings==true)
	{
		if(prop->bUseCompressor==true)
		{
			hr=pGraph->AddFilter(pCompressor,NULL);
			if (FAILED(hr))
			{
				UpdateError("The Compressor could not be added to the Filter Graph\n");
				Cleanup();
				return E_FAIL;
			}
		}
	}
	else
	{
		if(indexOfChosenCompressor!=-1)
		{
			hr=pGraph->AddFilter(CompressorArray[indexOfChosenCompressor],NULL);
			if (FAILED(hr))
			{
				UpdateError("The Compressor could not be added to the Filter Graph\n");
				Cleanup();
				return E_FAIL;
			}
		}
	}

	hr = AddFilterByCLSID(pGraph, CLSID_AviDest,    NULL, &pAviMux);
	if (FAILED(hr))
    {
        UpdateError("The AVI Mux could not be added to the Filter Graph\n");
		Cleanup();
        return E_FAIL;
    }
	
	hr = AddFilterByCLSID(pGraph, CLSID_FileWriter, NULL, &pFileWriter);
	if (FAILED(hr))
    {
        UpdateError("The File Writer could not be added to the Filter Graph\n");
		Cleanup();
        return E_FAIL;
    }

	//Add an audio renderer so that there is a default reference clock
	//For some reason SetSyncSource(NULL) won't work correctly otherwise
	hr = AddFilterByCLSID(pGraph, CLSID_AudioRender, NULL, &pAudioRenderer);
	if (FAILED(hr))
    {
        UpdateError("The Audio Renderer could not be added to the Filter Graph\n");
		Cleanup();
        return E_FAIL;
    }

	//Check if the name is valid
	if(prop->name==NULL)
	{
		UpdateError("The File Name is invalid\n");
		Cleanup();
        return E_FAIL;
	}

	LPOLESTR name;
	int cCharacters=strlen(prop->name)+1;

	if(cCharacters<=1)
	{
		UpdateError("The File Name is invalid\n");
		Cleanup();
        return E_FAIL;
	}

	//Make the directory
	char FilePath[1001];

	if(cCharacters>=1000)
	{
		UpdateError("The File Name is too long\n");
		Cleanup();
        return E_FAIL;
	}


	if(prop->name[0]=='/')
	{
		UpdateError("The File Name is invalid\n");
		Cleanup();
        return E_FAIL;
	}

	for(int i=0;i<1000;i++)
	{
		FilePath[i]=prop->name[i];
		if(FilePath[i]=='/')
		{
			FilePath[i+1]='\0';
			CreateDirectory(FilePath,NULL);
		}

		if(FilePath[i]=='\0')
			break;
	}

	//Check if the file exists
    WIN32_FIND_DATA fileData;
    HANDLE fHandle = FindFirstFile(prop->name, &fileData);
	if(fHandle!=INVALID_HANDLE_VALUE)
	{
		FindClose(fHandle);

		if(prop->bWriteOverIfExists==false)
		{
			UpdateError("The File already exists\n");
			Cleanup();
			return E_FAIL;
		}
	}


	//Get rid of old file (the File Writer just writes on top of the old file)
	DeleteFile(prop->name);

	//Tell the File Writer the file's name
	hr = pFileWriter->QueryInterface(IID_IFileSinkFilter, (void**)&pSink);
	if (FAILED(hr))
    {
        UpdateError("The File Writer's File Sink Interface could not be obtained\n");
		Cleanup();
        return E_FAIL;
    }

	name=(LPOLESTR)CoTaskMemAlloc(cCharacters*2);
	if(name==NULL)
	{
        UpdateError("Insufficient Memory to convert the name\n");
		Cleanup();
        return E_FAIL;
	}

	if(0==MultiByteToWideChar(CP_ACP, 0, prop->name, cCharacters, name, cCharacters))
	{
		UpdateError("Conversion of the File Name failed\n");
		CoTaskMemFree(name);
		Cleanup();
        return E_FAIL;
	}

	hr = pSink->SetFileName(name,NULL);//Originally name=L"MyVideoFile.avi"
	if (FAILED(hr))
    {
        UpdateError("The File Name could not be set\n");
		pSink->Release();
		pSink = NULL;
		CoTaskMemFree(name);
		Cleanup();
        return E_FAIL;
    }

	CoTaskMemFree(name);
	pSink->Release();
	pSink = NULL;

	//Connect the Filters
	if(prop->bUseTheseSettings==true)
	{
		if(prop->bUseCompressor==true)
		{
			hr=ConnectFilters(pGraph, pSource    , pCompressor);
			if (FAILED(hr))
			{
				UpdateError("The Source and the Compressor Filter failed to connect\n");
				Cleanup();
				return E_FAIL;
			}
			
			hr=ConnectFilters(pGraph, pCompressor, pAviMux);
			if (FAILED(hr))
			{
				UpdateError("The Compressor and the AVI Mux Filter failed to connect\n");
				Cleanup();
				return E_FAIL;
			}
		}
		else
		{
			hr=ConnectFilters(pGraph, pSource, pAviMux);
			if (FAILED(hr))
			{
				UpdateError("The Source and the AVI Mux Filter failed to connect\n");
				Cleanup();
				return E_FAIL;
			}
		}
	}
	else
	{
		if(indexOfChosenCompressor!=-1)
		{
			hr=ConnectFilters(pGraph, pSource, CompressorArray[indexOfChosenCompressor]);
			if (FAILED(hr))
			{
				UpdateError("The Source and the Compressor Filter failed to connect\n");
				Cleanup();
				return E_FAIL;
			}
			
			hr=ConnectFilters(pGraph, CompressorArray[indexOfChosenCompressor], pAviMux);
			if (FAILED(hr))
			{
				UpdateError("The Compressor and the AVI Mux Filter failed to connect\n");
				Cleanup();
				return E_FAIL;
			}
		}
		else
		{
			hr=ConnectFilters(pGraph, pSource, pAviMux);
			if (FAILED(hr))
			{
				UpdateError("The Source and the AVI Mux Filter failed to connect\n");
				Cleanup();
				return E_FAIL;
			}
		}
	}

	hr=ConnectFilters(pGraph, pAviMux    , pFileWriter);
	if (FAILED(hr))
    {
        UpdateError("The Avi Mux and the File Writer Filter failed to connect\n");
		Cleanup();
        return E_FAIL;
    }

    // Set the graph clock
    hr=pGraph->QueryInterface(IID_IMediaFilter, (void**)&pMediaFilter);
	if (FAILED(hr))
    {
        UpdateError("Could not obtain the Media Filter Interface\n");
		Cleanup();
        return E_FAIL;
    }

    hr=pMediaFilter->SetSyncSource(NULL);
	if (FAILED(hr))
    {
        UpdateError("Could not override the Reference Clock\n");
		pMediaFilter->Release();
		pMediaFilter=NULL;
		Cleanup();
        return E_FAIL;
    }

    pMediaFilter->Release();
	pMediaFilter=NULL;


    hr=pControl->Run();
	if (FAILED(hr))
    {
        UpdateError("Could not Run Graph\n");
		Cleanup();
        return E_FAIL;
    }

	HaveFilterGraph=true;

	return S_OK;
}

bool CFilterGraphManager::CheckVideoProperties(VideoProperties* prop)
{
	if(prop==NULL)
	{
		UpdateError("NULL pointer\n");
		return false;
	}

	if(prop->format<4 || prop->format>7)
	{
		UpdateError("Format not supported\n");
		return false;
	}

	if(prop->width<=0)
	{
		UpdateError("Width is not positive\n");
		return false;
	}

	if(prop->height==0)
	{
		UpdateError("Height is zero\n");
		return false;
	}


	return true;
}

bool CFilterGraphManager::FillCompressorProperties(IBaseFilter** ppFilter, CompressorProperties* prop)
{
	HRESULT hr = E_FAIL;
	IEnumPins             *pEnum            = NULL;
	IPin                  *pPin             = NULL;
	IAMVideoCompression   *pVidCompress     = NULL;
	IAMVfwCompressDialogs *pCompressDialogs = NULL;
	long Capabilities;

	if(prop==NULL || ppFilter==NULL || (*ppFilter)==NULL)
	{
		return false;
	}

	//Create the Pin Enumerator
	if(FAILED((*ppFilter)->EnumPins(&pEnum)))
	{
		return false;
	}


	// Find if the filter supports IAMVfwCompressDialogs (if any).
	prop->bHas_IAMVfwCompressDialogs_Interface=false;
	
	if(SUCCEEDED((*ppFilter)->QueryInterface(IID_IAMVfwCompressDialogs, (void**)&pCompressDialogs)))
	{
		// Found the interface.
		prop->bHas_IAMVfwCompressDialogs_Interface=true;
		
		//Fill in the relevant properties
		hr=pCompressDialogs->ShowDialog(VfwCompressDialog_QueryConfig,NULL);
		
		if(FAILED(hr))
		{
			pCompressDialogs->Release();
			pEnum->Release();
			return false;
		}
		
		if(hr==S_OK)
			prop->cdp.bConfigExists=true;
		else
			prop->cdp.bConfigExists=false;
		
		
		hr=pCompressDialogs->ShowDialog(VfwCompressDialog_QueryAbout,NULL);
		
		if(FAILED(hr))
		{
			pCompressDialogs->Release();
			pEnum->Release();
			return false;
		}
		
		if(hr==S_OK)
			prop->cdp.bAboutExists=true;
		else
			prop->cdp.bAboutExists=false;
		//Finished filling in properties 
		
		pCompressDialogs->Release();
	}

	// Find the pin that supports IAMVideoCompression (if any).
	pEnum->Reset();

	prop->bHas_IAMVideoCompression_Interface=false;
	while (true)
	{
		if(S_OK!=pEnum->Next(1, &pPin, NULL))
			break;

		if(SUCCEEDED(pPin->QueryInterface(IID_IAMVideoCompression, (void**)&pVidCompress)))
		{
			// Found the interface.
			prop->bHas_IAMVideoCompression_Interface=true;
			pPin->Release();

			//Fill in the relevant properties
			prop->vcp.pszDescription = NULL;
			prop->vcp.pszVersion     = NULL;

			hr=pVidCompress->GetInfo(
									NULL,&(prop->vcp.cbVersion),
									NULL,&(prop->vcp.cbDescription),
									&(prop->vcp.DefaultKeyFrameRate),
									&(prop->vcp.DefaultPFramesPerKey),
									&(prop->vcp.DefaultQuality),
									&Capabilities
									);
			if(FAILED(hr))
			{
				pVidCompress->Release();
				pEnum->Release();
				return false;
			}

			prop->vcp.bCAPCanQuality  = Capabilities & CompressionCaps_CanQuality;
			prop->vcp.bCAPCanCrunch   = Capabilities & CompressionCaps_CanCrunch;
			prop->vcp.bCAPCanKeyFrame = Capabilities & CompressionCaps_CanKeyFrame;
			prop->vcp.bCAPCanBFrame   = Capabilities & CompressionCaps_CanBFrame;
			prop->vcp.bCAPCanWindow   = Capabilities & CompressionCaps_CanWindow;

			if(prop->vcp.bCAPCanQuality)
			{
				hr=pVidCompress->get_Quality(&(prop->vcp.Quality));
				if(FAILED(hr) || prop->vcp.Quality<0)
					prop->vcp.Quality=prop->vcp.DefaultQuality;
			}

			if(prop->vcp.bCAPCanKeyFrame)
			{
				hr=pVidCompress->get_KeyFrameRate(&(prop->vcp.KeyFrameRate));
				if(FAILED(hr) || prop->vcp.KeyFrameRate<0)
					prop->vcp.KeyFrameRate=prop->vcp.DefaultKeyFrameRate;
			}

			if(prop->vcp.bCAPCanBFrame)
			{
				hr=pVidCompress->get_PFramesPerKeyFrame(&(prop->vcp.PFramesPerKey));
				if(FAILED(hr) || prop->vcp.PFramesPerKey<0)
					prop->vcp.PFramesPerKey=prop->vcp.DefaultPFramesPerKey;
			}
			
			if(prop->vcp.bCAPCanWindow)
			{
				hr=pVidCompress->get_WindowSize(&(prop->vcp.WindowSize));
				if(FAILED(hr))
					prop->vcp.WindowSize=-1;
			}



			prop->vcp.pszVersion=NULL;
			
			if(prop->vcp.cbVersion>=2)
			{
				prop->vcp.pszVersion = new WCHAR[prop->vcp.cbVersion/2];

				if(prop->vcp.pszVersion==NULL)
				{
					pVidCompress->Release();
					pEnum->Release();
					return false;
				}

				hr=pVidCompress->GetInfo(prop->vcp.pszVersion,NULL,NULL,NULL,
										 NULL,NULL,NULL,NULL);
				if(FAILED(hr))
				{
					delete [] prop->vcp.pszVersion;
					prop->vcp.pszVersion=NULL;

					pVidCompress->Release();
					pEnum->Release();
					return false;
				}

			}


			prop->vcp.pszDescription=NULL;

			if(prop->vcp.cbDescription>=2)
			{
				prop->vcp.pszDescription = new WCHAR[prop->vcp.cbDescription/2];

				if(prop->vcp.pszDescription==NULL)
				{
					if(prop->vcp.pszVersion!=NULL)
					{
						delete [] prop->vcp.pszVersion;
						prop->vcp.pszVersion=NULL;
					}

					pVidCompress->Release();
					pEnum->Release();
					return false;
				}

				hr=pVidCompress->GetInfo(NULL,NULL,prop->vcp.pszDescription,NULL,
										 NULL,NULL,NULL,NULL);
				if(FAILED(hr))
				{
					delete [] prop->vcp.pszDescription;
					prop->vcp.pszDescription=NULL;

					if(prop->vcp.pszVersion!=NULL)
					{
						delete [] prop->vcp.pszVersion;
						prop->vcp.pszVersion=NULL;
					}

					pVidCompress->Release();
					pEnum->Release();
					return false;
				}

			}
			//Finished filling in properties 

			pVidCompress->Release();
			break;
		}

		pPin->Release();
	}

	pEnum->Release();
	return true;
}

void CFilterGraphManager::ShowDialog(IBaseFilter** ppFilter, HWND hWndParent, bool bAbout, bool bConfig)
{
	HRESULT hr = E_FAIL;
	IAMVfwCompressDialogs *pCompressDialogs = NULL;
	CompressorProperties cp;

	if(ppFilter==NULL || (*ppFilter)==NULL)
		return;

	if(FillCompressorProperties(ppFilter,&cp)==false)
		return;

	if(cp.bHas_IAMVideoCompression_Interface==false)
		return;
	
	hr=(*ppFilter)->QueryInterface(IID_IAMVfwCompressDialogs, (void**) &pCompressDialogs);
	if(FAILED(hr))
		return;

	if(bAbout==true && cp.cdp.bAboutExists==true)
		pCompressDialogs->ShowDialog(VfwCompressDialog_About,hWndParent);

	if(bConfig==true && cp.cdp.bConfigExists==true)
		pCompressDialogs->ShowDialog(VfwCompressDialog_Config,hWndParent);

	pCompressDialogs->Release();
	pCompressDialogs=NULL;
	return;
}

void CFilterGraphManager::SetCompressorProperties(IBaseFilter** ppFilter, CompressorPropertiesToSet* prop)
{
	HRESULT hr = E_FAIL;
	IEnumPins             *pEnum            = NULL;
	IPin                  *pPin             = NULL;
	IAMVideoCompression   *pVidCompress     = NULL;
	CompressorProperties  cp;

	if(prop==NULL || ppFilter==NULL || (*ppFilter)==NULL)
		return;

	if(FillCompressorProperties(ppFilter,&cp)==false)
		return;

	if(cp.bHas_IAMVideoCompression_Interface==false)
		return;

	if(
		   prop->bSet_KeyFrameRate  == false
		&& prop->bSet_PFramesPerKey == false
		&& prop->bSet_Quality       == false
		&& prop->bSet_WindowSize    == false
	  )
		return;

	//Create the Pin Enumerator
	if(FAILED((*ppFilter)->EnumPins(&pEnum)))
		return;

	// Find the pin that supports IAMVideoCompression (if any).
	pEnum->Reset();

	while (true)
	{
		if(S_OK!=pEnum->Next(1, &pPin, NULL))
			break;

		if(SUCCEEDED(pPin->QueryInterface(IID_IAMVideoCompression, (void**)&pVidCompress)))
		{
			// Found the interface.
			pPin->Release();

			//Fill in the relevant properties
			if(cp.vcp.bCAPCanKeyFrame && prop->bSet_KeyFrameRate)
				pVidCompress->put_KeyFrameRate(prop->KeyFrameRate);
			
			if(cp.vcp.bCAPCanBFrame   && prop->bSet_PFramesPerKey)
				pVidCompress->put_PFramesPerKeyFrame(prop->PFramesPerKey);

			if(cp.vcp.bCAPCanQuality  && prop->bSet_Quality)
				pVidCompress->put_Quality(prop->Quality);

			if(cp.vcp.bCAPCanWindow   && prop->bSet_WindowSize)
				pVidCompress->put_WindowSize(prop->WindowSize);
			//Finished filling in properties 

			pVidCompress->Release();
			break;
		}

		pPin->Release();
	}

	pEnum->Release();

	return;
}

HRESULT CFilterGraphManager::AddFilterByCLSID(
    IGraphBuilder *pGraph,  // Pointer to the Filter Graph Manager.
    const GUID& clsid,      // CLSID of the filter to create.
    LPCWSTR wszName,        // A name for the filter.
    IBaseFilter **ppF)      // Receives a pointer to the filter.
{
    if (!pGraph || ! ppF) return E_POINTER;
    *ppF = 0;
    IBaseFilter *pF = 0;
    HRESULT hr = CoCreateInstance(clsid, 0, CLSCTX_INPROC_SERVER,
        IID_IBaseFilter, reinterpret_cast<void**>(&pF));
    if (SUCCEEDED(hr))
    {
        hr = pGraph->AddFilter(pF, wszName);
        if (SUCCEEDED(hr))
            *ppF = pF;
        else
            pF->Release();
    }
    return hr;
}

HRESULT CFilterGraphManager::GetUnconnectedPin(
    IBaseFilter *pFilter,   // Pointer to the filter.
    PIN_DIRECTION PinDir,   // Direction of the pin to find.
    IPin **ppPin)           // Receives a pointer to the pin.
{
    *ppPin = 0;
    IEnumPins *pEnum = 0;
    IPin *pPin = 0;
    HRESULT hr = pFilter->EnumPins(&pEnum);
    if (FAILED(hr))
    {
        return hr;
    }
    while (pEnum->Next(1, &pPin, NULL) == S_OK)
    {
        PIN_DIRECTION ThisPinDir;
        pPin->QueryDirection(&ThisPinDir);
        if (ThisPinDir == PinDir)
        {
            IPin *pTmp = 0;
            hr = pPin->ConnectedTo(&pTmp);
            if (SUCCEEDED(hr))  // Already connected, not the pin we want.
            {
                pTmp->Release();
            }
            else  // Unconnected, this is the pin we want.
            {
                pEnum->Release();
                *ppPin = pPin;
                return S_OK;
            }
        }
        pPin->Release();
    }
    pEnum->Release();
    // Did not find a matching pin.
    return E_FAIL;
}

HRESULT CFilterGraphManager::ConnectFilters(
    IGraphBuilder *pGraph, // Filter Graph Manager.
    IPin *pOut,            // Output pin on the upstream filter.
    IBaseFilter *pDest)    // Downstream filter.
{
    if ((pGraph == NULL) || (pOut == NULL) || (pDest == NULL))
    {
        return E_POINTER;
    }
	
	PIN_DIRECTION PinDir;
	pOut->QueryDirection(&PinDir);
	if(PinDir != PINDIR_OUTPUT)
		return E_FAIL;

    // Find an input pin on the downstream filter.
    IPin *pIn = 0;
    HRESULT hr = GetUnconnectedPin(pDest, PINDIR_INPUT, &pIn);
    if (FAILED(hr))
    {
        return hr;
    }
    // Try to connect them.
    hr = pGraph->Connect(pOut, pIn);
    pIn->Release();
    return hr;
}

HRESULT CFilterGraphManager::ConnectFilters(
    IGraphBuilder *pGraph, 
    IBaseFilter *pSrc, 
    IBaseFilter *pDest)
{
    if ((pGraph == NULL) || (pSrc == NULL) || (pDest == NULL))
    {
        return E_POINTER;
    }

    // Find an output pin on the first filter.
    IPin *pOut = 0;
    HRESULT hr = GetUnconnectedPin(pSrc, PINDIR_OUTPUT, &pOut);
    if (FAILED(hr)) 
    {
        return hr;
    }
    hr = ConnectFilters(pGraph, pOut, pDest);
    pOut->Release();
    return hr;
}

HRESULT CFilterGraphManager::FindVideoCompressor(IBaseFilter** ppFilter, char* FindFriendlyName)
{
	// Create the System Device Enumerator.
	HRESULT hr;
	ICreateDevEnum *pSysDevEnum = NULL;

	if(CoInit==false)
		return E_FAIL;

	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER,
		IID_ICreateDevEnum, (void **)&pSysDevEnum);
	if (FAILED(hr))
	{
		return hr;
	}
	
	// Obtain a class enumerator for the video compressor category.
	IEnumMoniker *pEnumCat = NULL;
	hr = pSysDevEnum->CreateClassEnumerator(CLSID_VideoCompressorCategory, &pEnumCat, 0);
	
	if (hr == S_OK) 
	{
		// Enumerate the monikers.
		IMoniker *pMoniker = NULL;
		ULONG cFetched;
		while(pEnumCat->Next(1, &pMoniker, &cFetched) == S_OK)
		{
			IPropertyBag *pPropBag;
			hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, 
				(void **)&pPropBag);
			if (SUCCEEDED(hr))
			{
				// To retrieve the filter's friendly name, do the following:
				VARIANT varName;
				VariantInit(&varName);
				hr = pPropBag->Read(L"FriendlyName", &varName, 0);
				if (SUCCEEDED(hr))
				{
					// Display the name in your UI somehow.
					if(varName.vt!=VT_BSTR)
					{
						//Name is not in the expected format
						
						/*
						printf("\nFindVideoCompressors():\n");
						printf("     Filter's 'Friendly Name' is not a wide string\n");
						printf("     Extra Information: %d\n",varName.vt);
						*/
					}
					else
					{
						char FriendlyName[256];
						sprintf(FriendlyName,"%ws",varName.bstrVal);
						if(0==strcmp(FriendlyName,FindFriendlyName))
						{
							/*
							printf("\nFindVideoCompressors():\n");
							printf("     Found Default Video Compressor\n");
							printf("     Extra Information: %s\n",FriendlyName);
							*/
							
							// To create an instance of the filter, do the following:
							hr = pMoniker->BindToObject(NULL, NULL, IID_IBaseFilter,
								(void**)ppFilter);
							// Now add the filter to the graph. 
							//Remember to release pFilter later.
							VariantClear(&varName);
							pPropBag->Release();
							pMoniker->Release();
							pEnumCat->Release();
							pSysDevEnum->Release();

							return S_OK;
						}
						
					}
				}
				VariantClear(&varName);
				pPropBag->Release();
			}
			pMoniker->Release();
		}
		pEnumCat->Release();
	}
	pSysDevEnum->Release();

	return E_FAIL;
}

