//------------------------------------------------------------------------------
// File: MyCaptureSource.h
//
// Desc: Header file for MyCaptureSource.cpp a modification of DirectShow's ball
//       source filter that requests bitmaps from the user
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Forward Declarations
//------------------------------------------------------------------------------
// The class managing the output pin
class CMyCaptureStream;


//------------------------------------------------------------------------------
// Class CMyCaptureSource
//
// This is the main class for the source filter. It inherits from
// CSource, the DirectShow base class for source filters.
//------------------------------------------------------------------------------
class CMyCaptureSource : public CSource
{
public:

    // Left from the dll implementation
    static CUnknown * WINAPI CreateInstance(LPUNKNOWN lpunk, HRESULT *phr);

    // Should create the object using this
    CMyCaptureSource(LPUNKNOWN lpunk, HRESULT *phr);

	void End();
	void SetDetails(int type, int width, int height, bool createBuffer);
	int  GetBufferSize();
	bool AddImage(BYTE* pData, int repeatTime);
}; // CMyCaptureSource


//------------------------------------------------------------------------------
// Class CMyCaptureStream
//
// This class implements the stream which is used to output the bitmap
// data from the source filter. It inherits from DirectShows's base
// CSourceStream class.
//------------------------------------------------------------------------------
class CMyCaptureStream : public CSourceStream
{

public:

    CMyCaptureStream(HRESULT *phr, CMyCaptureSource *pParent, LPCWSTR pPinName);
    ~CMyCaptureStream();

	//Checks if we have a new bitmap or we wish to end the video
	HRESULT DoBufferProcessingLoop(void);

    // Copies the bitmap into the supplied video frame
    HRESULT FillBuffer(IMediaSample *pms);

    // Ask for buffers of the size appropriate to the agreed media type
    HRESULT DecideBufferSize(IMemAllocator *pIMemAlloc,
                             ALLOCATOR_PROPERTIES *pProperties);

    // Set the agreed media type, and set up a temporary bitmap storage buffer
    HRESULT SetMediaType(const CMediaType *pMediaType);

    // Negotiate Media connection
    HRESULT CheckMediaType(const CMediaType *pMediaType);
    HRESULT GetMediaType(int iPosition, CMediaType *pmt);

    // Resets the stream time to zero
    HRESULT OnThreadCreate(void);

    // Quality control notifications sent to us (ignored)
    STDMETHODIMP Notify(IBaseFilter * pSender, Quality q);

	void End();
	void SetDetails(int type, int width, int height, bool createBuffer);
	int  GetBufferSize();
	bool AddImage(BYTE* pData, int repeatTime);

private:

    int m_iImageHeight;                 // The current image height
    int m_iImageWidth;                  // And current image width
    int m_iRepeatTime;                  // Time in msec between frames
	int m_iType;

    PALETTEENTRY m_Palette[256];        // The optimal palette for the image

    CCritSec m_cSharedState;            // Lock on members
    CRefTime m_rtSampleTime;            // The time stamp for each sample
	bool  bCreateBuffer;
	BYTE* m_bytes;
	long  m_sizeToCopy;
	bool bHasUsed;
	bool bWantToEnd;

    // set up the palette appropriately
    enum Colour {Red, Blue, Green, Yellow};
    HRESULT SetPaletteEntries(Colour colour);

}; // CMyCaptureStream
    
