#include "VideoCapture.h"
#include "FilterGraphManager.h"

void ResourceValues::Set(
		int s_IDD_COMPRESS_DIALOG,
		int s_IDC_COMBO,
		int s_IDC_CONFIGURE,
		int s_IDC_ABOUT,
		int s_IDOK,
		int s_IDC_SCROLLBAR,
		int s_IDC_QUALITY_TEXT,
		int s_IDC_VALUE
		)
{
	m_IDD_COMPRESS_DIALOG = s_IDD_COMPRESS_DIALOG;
	m_IDC_COMBO           = s_IDC_COMBO;
	m_IDC_CONFIGURE       = s_IDC_CONFIGURE;
	m_IDC_ABOUT           = s_IDC_ABOUT;
	m_IDOK                = s_IDOK;
	m_IDC_SCROLLBAR       = s_IDC_SCROLLBAR;
	m_IDC_QUALITY_TEXT    = s_IDC_QUALITY_TEXT;
	m_IDC_VALUE           = s_IDC_VALUE;

	return;
}



CompressorPropertiesToSet::CompressorPropertiesToSet()
{
	Name               = NULL;
	bSet_KeyFrameRate  = false;
	bSet_PFramesPerKey = false;
	bSet_Quality       = false;
	bSet_WindowSize    = false;
}

CompressorPropertiesToSet::~CompressorPropertiesToSet()
{
}



VideoCompressionProperties::VideoCompressionProperties()
{
	pszVersion     = NULL;
	pszDescription = NULL;
}

VideoCompressionProperties::~VideoCompressionProperties()
{
	Cleanup();
}

void VideoCompressionProperties::Cleanup()
{
	if(pszVersion != NULL)
	{
		delete [] pszVersion;
		pszVersion = NULL;
	}

	if(pszDescription != NULL)
	{
		delete [] pszDescription;
		pszDescription = NULL;
	}
	
	return;
}



CompressorProperties::CompressorProperties()
{
	bNameFilledIn=false;
	bInfoFilledIn=false;
}

CompressorProperties::~CompressorProperties()
{
	vcp.Cleanup();
}



CVideoCapture::CVideoCapture()
{
	cFGM=NULL;
}

CVideoCapture::~CVideoCapture()
{
	Destroy();
}

bool CVideoCapture::Create()
{
	if(cFGM!=NULL)
		return false;

	cFGM = new CFilterGraphManager;
	if(cFGM==NULL)
		return false;

	return true;
}

void CVideoCapture::Destroy()
{
	if(cFGM!=NULL)
	{
		Cleanup();
		CleanupArray();
		delete cFGM;
		cFGM=NULL;
	}

	return;
}

VC_HRESULT CVideoCapture::Init()
{
	if(cFGM==NULL)
		return E_FAIL;

	return ((CFilterGraphManager*)cFGM)->Init();
}

void CVideoCapture::Cleanup()
{
	if(cFGM==NULL)
		return;

	((CFilterGraphManager*)cFGM)->Cleanup();
	return;
}

VC_HRESULT CVideoCapture::CreateFilterGraph(VideoProperties* prop)
{
	if(cFGM==NULL)
		return E_FAIL;

	return ((CFilterGraphManager*)cFGM)->CreateFilterGraph(prop);
}

void CVideoCapture::End()
{
	if(cFGM==NULL)
		return;

	((CFilterGraphManager*)cFGM)->End();
	return;
}

int CVideoCapture::GetBufferSize()
{
	if(cFGM==NULL)
		return 0;

	return ((CFilterGraphManager*)cFGM)->GetBufferSize();
}

bool CVideoCapture::AddImage(VC_BYTE* pData, int repeatTime)
{
	if(cFGM==NULL)
		return false;

	return ((CFilterGraphManager*)cFGM)->AddImage(pData,repeatTime);
}

bool CVideoCapture::AddBitmap(VC_BYTE* pData, int repeatTime)
{
	if(cFGM==NULL)
		return false;

	return ((CFilterGraphManager*)cFGM)->AddBitmap(pData,repeatTime);
}

void CVideoCapture::DisplayCompressorProperties(CompressorProperties* cp)
{
	if(cFGM==NULL)
		return;

	((CFilterGraphManager*)cFGM)->DisplayCompressorProperties(cp);
	return;
}

VC_HRESULT CVideoCapture::GetAllCompressorInfo(CompressorProperties** pArray)
{
	if(cFGM==NULL)
		return E_FAIL;

	return ((CFilterGraphManager*)cFGM)->GetAllCompressorInfo(pArray);
}

bool CVideoCapture::GetError(char** pstr)
{
	if(cFGM==NULL)
		return false;

	if(pstr!=NULL)
		(*pstr)=((CFilterGraphManager*)cFGM)->ErrorString;

	return ((CFilterGraphManager*)cFGM)->Error;
}

void CVideoCapture::SetResourceValues(ResourceValues* rv)
{
	if(cFGM==NULL)
		return;

	((CFilterGraphManager*)cFGM)->SetResourceValues(rv);
	return;
}

void CVideoCapture::DisplayCompressorDialog(void* phWndParent, VideoProperties* prop)
{
	if(cFGM==NULL)
		return;

	if(phWndParent==NULL)
		((CFilterGraphManager*)cFGM)->DisplayCompressorDialog(NULL,prop);
	else
		((CFilterGraphManager*)cFGM)->DisplayCompressorDialog(*((HWND*)phWndParent),prop);

	return;
}

bool CVideoCapture::InitArray()
{
	if(cFGM==NULL)
		return false;

	return ((CFilterGraphManager*)cFGM)->InitArray(NULL,false);
}

void CVideoCapture::CleanupArray()
{
	if(cFGM==NULL)
		return;

	((CFilterGraphManager*)cFGM)->CleanupArray();
	return;
}

bool CVideoCapture::ReleaseArrayGraph()
{
	if(cFGM==NULL)
		return false;

	return ((CFilterGraphManager*)cFGM)->ReleaseArrayGraph();
}