#ifndef __QUATERNION__
#define __QUATERNION__

#include "Matrix.h"

class CQuaternion
{
public:
	
	CQuaternion();
	CQuaternion(double R, double X, double Y, double Z);
	CQuaternion(const CQuaternion& q);
	
	void        operator =( const CQuaternion&      q);
	CQuaternion operator -( const CQuaternion&      q) const;
	CQuaternion operator +( const CQuaternion&      q) const;
	CQuaternion operator *( const CQuaternion&      q) const;
	CQuaternion operator *( const CVector&          v) const;// *[0,v]
	CQuaternion operator *(       double       lambda) const;
	CQuaternion operator /(       double       lambda) const;
	void        operator +=(const CQuaternion&      q);
	void        operator -=(const CQuaternion&      q);
	void        operator *=(const CQuaternion&      q);
	void        operator *=(const CVector&          v);// *=[0,v]
	void        operator *=(      double       lambda);
	void        operator /=(      double       lambda);
	
	double      Length() const;
	void        Normalize();

	//conjugate of [r,v] = [r,-v]
	void        Conjugate();
	CQuaternion ReturnConjugate() const;

	double e[4];
	//e[0]=r
	//e[1]=i
	//e[2]=j
	//e[3]=k
	//i*i=j*j=k*k=-1, ij=k, jk=i, ki=j
	//q1*q2 is equivalent to applying the rotation q2 then q1
};

CQuaternion operator *(      double   lambda, const CQuaternion& q);
CQuaternion operator *(const CVector&      v, const CQuaternion& q);// [0,v]*q

//The matrix and quaternion will be normalized
//(In the case of matrices it will orthogonalize the columns and ensure det=1)
void MatrixToQuaternion        (const CMatrix&     A, CQuaternion& q);
void QuaternionToMatrix        (const CQuaternion& q, CMatrix&     A);

void RotationVectorToQuaternion(const CVector&     w, CQuaternion& q);
void QuaternionToRotationVector(const CQuaternion& q, CVector&     w);

//r=0, i=v1, j=v2, k=v3
void VectorToQuaternion        (const CVector&     v, CQuaternion& q);
void QuaternionToVector        (const CQuaternion& q, CVector&     v);

//Axis will be normalized by function (angle is assumed to be in radians)
void AngleAxisToQuaternion     (double angle, CVector n, CQuaternion& q);

#endif

//Not yet implemented an explicit rotation function x'=qxq* for x = vector/matrix
//
//unit quaternions represent rotations
//both q and -q represent the same rotation so make sure they're
//both on the same 4d hemisphere before interpolating

