#include "LinearProgramSolver.h"

//Link glpk49.lib

extern "C" {
#include "../glpk49Include/glpk.h"
}

int glpkSimplexAlgorithm(const CMatrixList& mData, CMatrixList& Result)
{
	double  ZeroTolerance = 0.0000001;
	int     i,j;
	int     numOfNonZeroElements;
	int     mDataX,mDataY;
    LPX*    lp;
    int*    ia;
	int*    ja;  
	double* ar;

	mData.getDimensions(&mDataY, &mDataX);

	//Matrix not big enough
	if(mDataX<3 || mDataY<2)
		return -4;
	
	if(!Result.setDimensions(mDataX-2,1))
		return -1;//ran out of memory

	Result .Fill(0);

	ia = NULL;
	ia = new    int[(mDataX-2)*(mDataY-1)+1];
	if(ia==NULL)
		return -1;//ran out of memory

	ja = NULL;
	ja = new    int[(mDataX-2)*(mDataY-1)+1];
	if(ja==NULL)
	{
		delete[] ia;
		return -1;//ran out of memory
	}

	ar = NULL;
	ar = new double[(mDataX-2)*(mDataY-1)+1];
	if(ar==NULL)
	{
		delete[] ia;
		delete[] ja;
		return -1;//ran out of memory
	}

	lp = lpx_create_prob();

	if(-0.5<mData.GetValue(1,mDataX-1) && mData.GetValue(1,mDataX-1)<0.5)
		lpx_set_obj_dir(lp, LPX_MIN);
	if(mData.GetValue(1,mDataX-1)<-0.1)
		lpx_set_obj_dir(lp, LPX_MIN);
	if(mData.GetValue(1,mDataX-1)>0.1)
		lpx_set_obj_dir(lp, LPX_MAX);

	lpx_add_rows(lp, mDataY-1);

	for(i=1;i<=mDataY-1;i++)
	{
		if(-0.5<mData.GetValue(i+1,mDataX-1) && mData.GetValue(i+1,mDataX-1)<0.5)
		{
			//Equals
			lpx_set_row_bnds(lp,i,LPX_FX,mData.GetValue(i+1,mDataX),0.0);
		}
		else
		{
			if(mData.GetValue(i+1,mDataX-1)<-0.1)
			{
				//>=
				lpx_set_row_bnds(lp,i,LPX_LO,mData.GetValue(i+1,mDataX),0.0);
			}

			if(mData.GetValue(i+1,mDataX-1)>0.1)
			{
				//<=
				lpx_set_row_bnds(lp,i,LPX_UP,0.0,mData.GetValue(i+1,mDataX));
			}
		}
	}

	lpx_add_cols(lp, mDataX-2);

	for(i=1;i<=mDataX-2;i++)
		lpx_set_col_bnds(lp,i,LPX_LO,0.0,0.0);

	if(-0.1<mData.GetValue(1,mDataX-1) && mData.GetValue(1,mDataX-1)<0.1)
	{
		for(i=1;i<=mDataX-2;i++)
			lpx_set_obj_coef(lp,i,0.0);
	}
	else
	{
		for(i=1;i<=mDataX-2;i++)
			lpx_set_obj_coef(lp,i,mData.GetValue(1,i));
	}

	numOfNonZeroElements = 0;
	for(i=1;i<=mDataY-1;i++)
	for(j=1;j<=mDataX-2;j++)
	{
		double result;

		result = mData.GetValue(i+1,j);

		//Check if result is zero 
		if(-ZeroTolerance<result && result<ZeroTolerance)
			continue;

		numOfNonZeroElements++;
		ia[numOfNonZeroElements] = i;
		ja[numOfNonZeroElements] = j;
		ar[numOfNonZeroElements] = result;
	}

	lpx_load_matrix(lp, numOfNonZeroElements, ia, ja, ar);
	
	if(lpx_simplex(lp)!=LPX_E_OK)
	{
		lpx_delete_prob(lp);
		delete[] ia;
		delete[] ja;
		delete[] ar;
		return -6;
	}

	if(lpx_get_status(lp)==LPX_UNBND)
	{
		lpx_delete_prob(lp);
		delete[] ia;
		delete[] ja;
		delete[] ar;
		return -3;
	}

	if(lpx_get_status(lp)!=LPX_OPT)
	{
		lpx_delete_prob(lp);
		delete[] ia;
		delete[] ja;
		delete[] ar;
		return -2;
	}

	for(i=1;i<=mDataX-2;i++)
		Result.SetValue(i,1,lpx_get_col_prim(lp,i));
	
	lpx_delete_prob(lp);
	delete[] ia;
	delete[] ja;
	delete[] ar;

	return 1;
}