#ifndef __3DS_OBJECT__
#define __3DS_OBJECT__

#include "MotionData.h"


class CVertex
{
public:
	//Position of a vertex in model space
	float x,y,z;
	//Color of vertex
	float r,g,b;

	//To be filled with the average of face normals
	//for lighting purposes (done by Load3dsFile())
	CVector Normal;
};


class CTriFace
{
public:
	//Specifies three vertices that make up the triangle face
	//by storing three indices to the vertList[] array
	int index[3];

	//To be filled with the outward face normal (done by Load3dsFile())
	CVector Normal;
};


class C3dObject
{
public:

	C3dObject();
	~C3dObject();

	//Total number of vertices and triangle faces
	int iNumOfVerts;
	int iNumOfFaces;

	//Array of vertices and triangle faces
	CVertex*  vertList;
	CTriFace* faceList;

	//Name of the object (filled in by Load3dsFile())
	char name[21];

	//Simulation Properties
	CMotionData data;

	//Transform vertList[i]'s position and normal values
	//with the orientation matrix and position vector in the data member
	//(copies the rgb data)
	CVertex Transform(int i);
	
	//Transform vertList[i]'s position
	//with the orientation matrix and position vector in the data member
	CVector TransformToWorld(int i);
};

//Moves the centre of mass to the origin in model space 
//and fills in the inertia tensor for the object.
void CalculateSolidInertiaAndCOM(C3dObject* pMesh);
void CalculateHollowInertiaAndCOM(C3dObject* pMesh);

//See MotionData.cpp for implementations
//Updates the simulation (moves it forward by one time step)
void UpdateMotionData(C3dObject* pMesh, int NoOfObjects);
//Allocates Memory for the global variables (returns 0 if it fails)
int  InitGlobalVariables(int numOfVertices);
//Resleases Memory held by global variables
void ReleaseGlobalVariables();

#endif

