#include "3ds_loader.h"
#include <stdio.h>


//Fill in vertex and face normals to be used for lighting
void CalculateNormals(C3dObject* p3dObject)
{
	int i,j;
	int k0,k1,k2;
	CVector u,v;
	CVector n;

	//Zero Out Normals
	for(i=0;i<p3dObject->iNumOfVerts;i++)
		p3dObject->vertList[i].Normal*=0;
	for(i=0;i<p3dObject->iNumOfFaces;i++)
		p3dObject->faceList[i].Normal*=0;

	//Fill in Face Normals
	//Fill in Vertex Normals with the sum of their adjacent Face Normals
	for(i=0;i<p3dObject->iNumOfFaces;i++)
	{
		for(j=0;j<3;j++)
		{
			k0=j;
			k1=(j+1)%3;
			k2=(j+2)%3;

			u.e[0] = p3dObject->vertList[p3dObject->faceList[i].index[k1]].x
				    -p3dObject->vertList[p3dObject->faceList[i].index[k0]].x;
			u.e[1] = p3dObject->vertList[p3dObject->faceList[i].index[k1]].y
				    -p3dObject->vertList[p3dObject->faceList[i].index[k0]].y;
			u.e[2] = p3dObject->vertList[p3dObject->faceList[i].index[k1]].z
				    -p3dObject->vertList[p3dObject->faceList[i].index[k0]].z;

			v.e[0] = p3dObject->vertList[p3dObject->faceList[i].index[k2]].x
				    -p3dObject->vertList[p3dObject->faceList[i].index[k0]].x;
			v.e[1] = p3dObject->vertList[p3dObject->faceList[i].index[k2]].y
				    -p3dObject->vertList[p3dObject->faceList[i].index[k0]].y;
			v.e[2] = p3dObject->vertList[p3dObject->faceList[i].index[k2]].z
				    -p3dObject->vertList[p3dObject->faceList[i].index[k0]].z;
			
			n=u.Cross(v);

			if(n.Length()>p3dObject->faceList[i].Normal.Length())
				p3dObject->faceList[i].Normal=n;
		}

		//Normalize the Face Normal
		p3dObject->faceList[i].Normal.Normalize();

		//Add to total of Vertex Normals
		for(j=0;j<3;j++)
			p3dObject->vertList[p3dObject->faceList[i].index[j]].Normal+=p3dObject->faceList[i].Normal;
	}

	//Normalize the Vertex Normals;
	for(i=0;i<p3dObject->iNumOfVerts;i++)
		p3dObject->vertList[i].Normal.Normalize();

	return;
}


int Load3dsFile(C3dObject* p3dObject, char* pFilename)
{
	FILE* pFileToLoad;
    unsigned short usChunkID;
	unsigned int   uiChunkLength;
	unsigned short usNumInList;
	bool           bHaveVertInfo;
	bool           bHaveFaceInfo;

	pFileToLoad   = NULL;
	bHaveVertInfo = false;
	bHaveFaceInfo = false;

	pFileToLoad=fopen(pFilename,"rb");
	if(pFileToLoad==NULL)
		return 0;

	fread(&usChunkID     ,2,1,pFileToLoad);
	fread(&uiChunkLength ,4,1,pFileToLoad);

	while(1)
	{
		char c;
		int  i;

		fread(&usChunkID     ,2,1,pFileToLoad);
		fread(&uiChunkLength ,4,1,pFileToLoad);

		switch(usChunkID)
		{
		case 0x4d4d://MAIN chunk
			break;

		case 0x3d3d://3D EDITOR chunk
			break;

		case 0x4000://OBJECT chunk
			//Read Name of Object
			i=0;
			do
			{
				fread(&c,1,1,pFileToLoad);
				if(c!='\0')
				{
					if(i<20)
					{
						p3dObject->name[i]=c;
						i++;
					}
				}
				else
				{
					p3dObject->name[i]='\0';
					break;
				}
			}while(1);
			break;
		
		case 0x4100://TRIANGULAR MESH chunk
			break;

		case 0x4110://VERTEX LIST chunk
			fread(&usNumInList,2,1,pFileToLoad);
			p3dObject->iNumOfVerts=usNumInList;

			p3dObject->vertList=new CVertex[usNumInList];
			if(p3dObject->vertList==NULL)
			{
				printf("\nError\n");
			}

			for(i=0;i<usNumInList;i++)
			{
				fread(&(p3dObject->vertList[i].x),sizeof(float),1,pFileToLoad);
				fread(&(p3dObject->vertList[i].y),sizeof(float),1,pFileToLoad);
				fread(&(p3dObject->vertList[i].z),sizeof(float),1,pFileToLoad);
			}

			bHaveVertInfo=true;
			break;

		case 0x4120://FACES LIST chunk
			unsigned short int j1,j2,j3;
			fread(&usNumInList,2,1,pFileToLoad);
			p3dObject->iNumOfFaces=usNumInList;
			
			p3dObject->faceList=new CTriFace[usNumInList];
			if(p3dObject->faceList==NULL)
			{
				printf("\nError\n");
			}

			for(i=0;i<usNumInList;i++)
			{
				fread(&j1,sizeof(unsigned short),1,pFileToLoad);
				fread(&j2,sizeof(unsigned short),1,pFileToLoad);
				fread(&j3,sizeof(unsigned short),1,pFileToLoad);
				fseek(pFileToLoad,sizeof(unsigned short),SEEK_CUR);

				p3dObject->faceList[i].index[0] = j1;
				p3dObject->faceList[i].index[1] = j2;
				p3dObject->faceList[i].index[2] = j3;
			}

			bHaveFaceInfo=true;
			break;


		default:
			//Skip whole chunk
			fseek(pFileToLoad,uiChunkLength-6,SEEK_CUR);
		}

		if(bHaveVertInfo && bHaveFaceInfo)
			break;
	}

	fclose(pFileToLoad);

	CalculateNormals(p3dObject);

	return 1;
}

