#include "DifferentialEquation.h"

//Runge-Kutta of order 4
void ode(CVectorList* x_start, CVectorList* x_final,
		 double       t_start, double       t_final,
		 int          steps,   CalcXdot     f)
{
	int i;
	double  h;
	double  t;
	double       L[5];
	CVectorList* V[5];
	CVectorList x0,x1,k1,k2,k3,k4;


	//fill x_final with 0 as default
	x_final->Fill(0);

	//Allocate memory for Vector Lists
	i=(x_start->dimension>x_final->dimension)?x_final->dimension:x_start->dimension;
	x0.setDimension(i);
	x1.setDimension(i);
	k1.setDimension(i);
	k2.setDimension(i);
	k3.setDimension(i);
	k4.setDimension(i);

	//Initialize
	h = (t_final-t_start)/steps;
	
	x1=*x_start;

	for(i=0;i<steps;i++)
	{
		t=t_start+i*h;
		x0=x1;

		//Calc k1 = f(x0,t0)
		f(&k1,&x0,t);

		//Calc k2 = f(x0+k1*h/2,t0+h/2)
		L[0]=1;   V[0]=&x1;
		L[1]=h/2; V[1]=&k1;
		x0.LinearAddition(V,L,2);
		f(&k2,&x0,t+h/2);

		//Calc k3 = f(x0+k2*h/2,t0+h/2)
		L[0]=1;   V[0]=&x1;
		L[1]=h/2; V[1]=&k2;
		x0.LinearAddition(V,L,2);
		f(&k3,&x0,t+h/2);

		//Calc k4 = f(x0+k3*h,t0+h)
		L[0]=1;   V[0]=&x1;
		L[1]=h;   V[1]=&k3;
		x0.LinearAddition(V,L,2);
		f(&k4,&x0,t+h);

		//Calc x1=x0+h/6*(k1+2*k2+2*k3+k4)
		L[0]=1;   V[0]=&x1;
		L[1]=h/6; V[1]=&k1;
		L[2]=h/3; V[2]=&k2;
		L[3]=h/3; V[3]=&k3;
		L[4]=h/6; V[4]=&k4;
		x1.LinearAddition(V,L,5);
	}

	*x_final=x1;
	return;
}
