#include "Quaternion.h"

CQuaternion::CQuaternion()
{
	e[0] = 1.0;
	e[1] = 0.0;
	e[2] = 0.0;
	e[3] = 0.0;
}


CQuaternion::CQuaternion(double R, double X, double Y, double Z)
{
	e[0] = R;
	e[1] = X;
	e[2] = Y;
	e[3] = Z;
}


CQuaternion::CQuaternion(const CQuaternion& q)
{
	e[0] = q.e[0];
	e[1] = q.e[1];
	e[2] = q.e[2];
	e[3] = q.e[3];
}


void CQuaternion::operator =(const CQuaternion& q)
{
	e[0] = q.e[0];
	e[1] = q.e[1];
	e[2] = q.e[2];
	e[3] = q.e[3];
}


CQuaternion CQuaternion::operator -(const CQuaternion& q) const
{
	return CQuaternion(e[0] - q.e[0], e[1] - q.e[1], e[2] - q.e[2], e[3] - q.e[3]);
}


CQuaternion CQuaternion::operator +(const CQuaternion& q) const
{
	return CQuaternion(e[0] + q.e[0], e[1] + q.e[1], e[2] + q.e[2], e[3] + q.e[3]);
}


CQuaternion CQuaternion::operator *(const CQuaternion& q) const
{
	return CQuaternion(
		e[0]*q.e[0] - e[1]*q.e[1] - e[2]*q.e[2] - e[3]*q.e[3],
		e[0]*q.e[1] + e[1]*q.e[0] + e[2]*q.e[3] - e[3]*q.e[2],
		e[0]*q.e[2] - e[1]*q.e[3] + e[2]*q.e[0] + e[3]*q.e[1],
		e[0]*q.e[3] + e[1]*q.e[2] - e[2]*q.e[1] + e[3]*q.e[0]);
}


CQuaternion CQuaternion::operator *(const CVector& v) const
{
	return CQuaternion(
		             -e[1]*v.e[0] - e[2]*v.e[1] - e[3]*v.e[2],
		e[0]*v.e[0]               + e[2]*v.e[2] - e[3]*v.e[1],
		e[0]*v.e[1] - e[1]*v.e[2]               + e[3]*v.e[0],
		e[0]*v.e[2] + e[1]*v.e[1] - e[2]*v.e[0]              );
}


CQuaternion CQuaternion::operator *(double lambda) const
{
	return CQuaternion(e[0] * lambda, e[1] * lambda, e[2] * lambda, e[3] * lambda);
}


CQuaternion CQuaternion::operator /(double lambda) const
{
	return CQuaternion(e[0] / lambda, e[1] / lambda, e[2] / lambda, e[3] / lambda);
}


void CQuaternion::operator +=(const CQuaternion& q)
{
	e[0] += q.e[0];
	e[1] += q.e[1];
	e[2] += q.e[2];
	e[3] += q.e[3];
}


void CQuaternion::operator -=(const CQuaternion& q)
{
	e[0] -= q.e[0];
	e[1] -= q.e[1];
	e[2] -= q.e[2];
	e[3] -= q.e[3];
}


void CQuaternion::operator *=(const CQuaternion& q)
{
	double temp0,temp1,temp2,temp3;
	
	temp0 = e[0]*q.e[0] - e[1]*q.e[1] - e[2]*q.e[2] - e[3]*q.e[3];
	temp1 = e[0]*q.e[1] + e[1]*q.e[0] + e[2]*q.e[3] - e[3]*q.e[2];
	temp2 = e[0]*q.e[2] - e[1]*q.e[3] + e[2]*q.e[0] + e[3]*q.e[1];
	temp3 =	e[0]*q.e[3] + e[1]*q.e[2] - e[2]*q.e[1] + e[3]*q.e[0];

	e[0]  = temp0;
	e[1]  = temp1;
	e[2]  = temp2;
	e[3]  = temp3;

	return;
}


void CQuaternion::operator *=(const CVector& v)
{
	double temp0,temp1,temp2,temp3;

	temp0 =              -e[1]*v.e[0] - e[2]*v.e[1] - e[3]*v.e[2];
	temp1 = e[0]*v.e[0]               + e[2]*v.e[2] - e[3]*v.e[1];
	temp2 =	e[0]*v.e[1] - e[1]*v.e[2]               + e[3]*v.e[0];
	temp3 = e[0]*v.e[2] + e[1]*v.e[1] - e[2]*v.e[0]              ;

	e[0]  = temp0;
	e[1]  = temp1;
	e[2]  = temp2;
	e[3]  = temp3;

	return;
}


void CQuaternion::operator *=(double lambda)
{
	e[0] *= lambda;
	e[1] *= lambda;
	e[2] *= lambda;
	e[3] *= lambda;
}


void CQuaternion::operator /=(double lambda)
{
	e[0] /= lambda;
	e[1] /= lambda;
	e[2] /= lambda;
	e[3] /= lambda;
}


double CQuaternion::Length() const
{
	return sqrt(e[0]*e[0] + e[1]*e[1] + e[2]*e[2] + e[3]*e[3]);
}


void CQuaternion::Normalize()
{
	double length;
	
	length = Length();
	
	if(length == 0.0f)
		return;
	
	e[0] = e[0]/length;
	e[1] = e[1]/length;
	e[2] = e[2]/length;
	e[3] = e[3]/length;
	
	return;
}


void CQuaternion::Conjugate()
{
	//e[0] remains the same
	e[1] = -e[1];
	e[2] = -e[2];
	e[3] = -e[3];

	return;
}


CQuaternion CQuaternion::ReturnConjugate() const
{
	return CQuaternion(e[0], -e[1], -e[2], -e[3]);
}


CQuaternion operator *(double lambda, const CQuaternion& q)
{
	return CQuaternion(q.e[0]*lambda, q.e[1]*lambda, q.e[2]*lambda, q.e[3]*lambda);
}


CQuaternion operator *(const CVector& v, const CQuaternion& q)
{
	return CQuaternion(
		-v.e[0]*q.e[1] - v.e[1]*q.e[2] - v.e[2]*q.e[3],
		 v.e[0]*q.e[0] + v.e[1]*q.e[3] - v.e[2]*q.e[2],
		-v.e[0]*q.e[3] + v.e[1]*q.e[0] + v.e[2]*q.e[1],
		 v.e[0]*q.e[2] - v.e[1]*q.e[1] + v.e[2]*q.e[0]);
}


void MatrixToQuaternion(const CMatrix& A, CQuaternion& q)
{
	int i,j;
	double tolerence = 0.0001;
	double length[3];
	double trace;
	double temp;
	CVector v[3];
	CMatrix B;

	//Orthogonalize Matrix
	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			v[j].e[i]=A.M[i][j];

	for(i=0;i<3;i++)
	{
		length[i]=v[i].Length();
		if(length[i]<tolerence)
			return;//Failed to normalize columns
		v[i]/=length[i];
	}

	v[2]=v[0].Cross(v[1]);
	v[1]=v[2].Cross(v[0]);
	v[0].Normalize();
	v[1].Normalize();
	v[2].Normalize();

	for(i=0;i<3;i++)
		for(j=0;j<3;j++)
			B.M[i][j]=v[j].e[i];
	

	//Calculate quaternion
	trace = B.M[0][0]+B.M[1][1]+B.M[2][2];

	if(trace>=0)
	{
		temp   = sqrt(trace+1);
		q.e[0] = temp/2;
		temp   = 0.5/temp;
		q.e[1] = (B.M[2][1]-B.M[1][2])*temp;
		q.e[2] = (B.M[0][2]-B.M[2][0])*temp;
		q.e[3] = (B.M[1][0]-B.M[0][1])*temp;
		
		return;
	}

	if(B.M[0][0]>B.M[1][1] && B.M[0][0]>B.M[2][2])
	{
		temp   = sqrt(B.M[0][0]-B.M[1][1]-B.M[2][2]+1);
		q.e[1] = temp/2;
		temp   = 0.5/temp;
		q.e[0] = (B.M[2][1]-B.M[1][2])*temp;
		q.e[2] = (B.M[0][1]+B.M[1][0])*temp;
		q.e[3] = (B.M[0][2]+B.M[2][0])*temp;
		
		return;
	}

	if(B.M[1][1]>B.M[2][2])
	{
		temp   = sqrt(B.M[1][1]-B.M[0][0]-B.M[2][2]+1);
		q.e[2] = temp/2;
		temp   = 0.5/temp;
		q.e[0] = (B.M[0][2]-B.M[2][0])*temp;
		q.e[1] = (B.M[0][1]+B.M[1][0])*temp;
		q.e[3] = (B.M[1][2]+B.M[2][1])*temp;
		
		return;
	}

	//B.M[2][2] largest diagonal element
	{
		temp   = sqrt(B.M[2][2]-B.M[0][0]-B.M[1][1]+1);
		q.e[3] = temp/2;
		temp   = 0.5/temp;
		q.e[0] = (B.M[1][0]-B.M[0][1])*temp;
		q.e[1] = (B.M[0][2]+B.M[2][0])*temp;
		q.e[2] = (B.M[1][2]+B.M[2][1])*temp;
		
		return;
	}

	return;
}


void QuaternionToMatrix(const CQuaternion& q, CMatrix& A)
{
	double      length;
	CQuaternion n;

	length = q.Length();
	if(length==0)
		return;

	n.e[0] = q.e[0]/length;
	n.e[1] = q.e[1]/length;
	n.e[2] = q.e[2]/length;
	n.e[3] = q.e[3]/length;

	//column 0
	A.M[0][0] = 1-2*(n.e[2]*n.e[2]+n.e[3]*n.e[3]);
	A.M[1][0] =   2*(n.e[1]*n.e[2]+n.e[0]*n.e[3]);
	A.M[2][0] =   2*(n.e[1]*n.e[3]-n.e[0]*n.e[2]);

	//column 1
	A.M[0][1] =   2*(n.e[1]*n.e[2]-n.e[0]*n.e[3]);
	A.M[1][1] = 1-2*(n.e[1]*n.e[1]+n.e[3]*n.e[3]);
	A.M[2][1] =   2*(n.e[2]*n.e[3]+n.e[0]*n.e[1]);

	//column 2
	A.M[0][2] =   2*(n.e[1]*n.e[3]+n.e[0]*n.e[2]);
	A.M[1][2] =   2*(n.e[2]*n.e[3]-n.e[0]*n.e[1]);
	A.M[2][2] = 1-2*(n.e[1]*n.e[1]+n.e[2]*n.e[2]);

	return;
}


void RotationVectorToQuaternion(const CVector& w, CQuaternion& q)
{
	double tolerance=0.00001;
	double length;

	length = w.Length();
	q.e[0] = cos(length/2);
	q.e[1] = sin(length/2)*w.e[0];
	q.e[2] = sin(length/2)*w.e[1];
	q.e[3] = sin(length/2)*w.e[2];

	if(length<tolerance)
		return;

	q.e[1]/=length;
	q.e[2]/=length;
	q.e[3]/=length;

	return;
}


void QuaternionToRotationVector(const CQuaternion& q, CVector& w)
{
	double theta;
	double c,s;
	CQuaternion n;

	n = q;
	n.Normalize();

	w.e[0] = n.e[1];
	w.e[1] = n.e[2];
	w.e[2] = n.e[3];

	c = n.e[0];
	s = w.Length();
	if(s==0)
		return;

	theta = atan2(s,c)*2;

	w=w*theta/s;

	return;
}


void VectorToQuaternion(const CVector& v, CQuaternion& q)
{
	q.e[0] = 0;
	q.e[1] = v.e[0];
	q.e[2] = v.e[1];
	q.e[3] = v.e[2];

	return;
}


void QuaternionToVector(const CQuaternion& q, CVector& v)
{
	//Ignore q.e[0]
	v.e[0] = q.e[1];
	v.e[1] = q.e[2];
	v.e[2] = q.e[3];

	return;
}


void AngleAxisToQuaternion(double angle, CVector n, CQuaternion& q)
{
	double s;

	s = sin(angle/2);
	n.Normalize();

	q.e[0] = cos(angle/2);
	q.e[1] = s*n.e[0];
	q.e[2] = s*n.e[1];
	q.e[3] = s*n.e[2];

	return;
}

