#include"Vector.h"


CVector::CVector()
{
	e[0] = 1.0;
	e[1] = 0.0;
	e[2] = 0.0;
}


CVector::CVector(double X, double Y, double Z)
{
	e[0] = X;
	e[1] = Y;
	e[2] = Z;
}


CVector::CVector(const CVector& v)
{
	e[0] = v.e[0];
	e[1] = v.e[1];
	e[2] = v.e[2];
}


void CVector::operator =(const CVector& v)
{
	e[0] = v.e[0];
	e[1] = v.e[1];
	e[2] = v.e[2];
}


CVector CVector::operator -(const CVector& v) const
{
	return CVector(e[0] - v.e[0], e[1] - v.e[1], e[2] - v.e[2]);
}


CVector CVector::operator +(const CVector& v) const
{
	return CVector(e[0] + v.e[0], e[1] + v.e[1], e[2] + v.e[2]);
}


CVector CVector::operator *(double lambda) const
{
	return CVector(e[0] * lambda, e[1] * lambda, e[2] * lambda);
}


CVector CVector::operator /(double lambda) const
{
	return CVector(e[0] / lambda, e[1] / lambda, e[2] / lambda);
}


void CVector::operator +=(const CVector& v)
{
	e[0] += v.e[0];
	e[1] += v.e[1];
	e[2] += v.e[2];
}


void CVector::operator -=(const CVector& v)
{
	e[0] -= v.e[0];
	e[1] -= v.e[1];
	e[2] -= v.e[2];
}


void CVector::operator *=(double lambda)
{
	e[0] *= lambda;
	e[1] *= lambda;
	e[2] *= lambda;
}


void CVector::operator /=(double lambda)
{
	e[0] /= lambda;
	e[1] /= lambda;
	e[2] /= lambda;
}


CVector CVector::Cross(const CVector& v) const
{
	return CVector(e[1]*v.e[2]-e[2]*v.e[1], e[2]*v.e[0]-e[0]*v.e[2], e[0]*v.e[1]-e[1]*v.e[0]);
}


double CVector::Dot(const CVector& v) const
{
	return e[0]*v.e[0] + e[1]*v.e[1] + e[2]*v.e[2];
}


double CVector::Length() const
{
	return sqrt(e[0]*e[0] + e[1]*e[1] + e[2]*e[2]);
}


void CVector::Normalize()
{
	double length;
	
	length = Length();
	
	if(length == 0.0f)
		return;
	
	e[0] = e[0]/length;
	e[1] = e[1]/length;
	e[2] = e[2]/length;
	
	return;
}


CVector operator *(double lambda, const CVector& v)
{
	return CVector(lambda * v.e[0], lambda * v.e[1], lambda * v.e[2]);
}

